<?php /*

 Composr
 Copyright (c) Christopher Graham, 2004-2024

 See docs/LICENSE.md for full licensing information.

*/

/**
 * @license    http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright  Christopher Graham
 * @package    cms_homesite_tracker
 */

/**
 * Hook class.
 */
class Hook_addon_registry_cms_homesite_tracker
{
    /**
     * Get a list of file permissions to set.
     *
     * @param  boolean $runtime Whether to include wildcards represented runtime-created chmoddable files
     * @return array File permissions to set
     */
    public function get_chmod_array(bool $runtime = false) : array
    {
        return [];
    }

    /**
     * Get the current version of this addon (usually software major, software minor, addon build).
     * Put the comment "// addon_version_auto_update" to the right of the return if you want release tools to automatically update this according to software version and find_addon_effective_mtime.
     *
     * @return SHORT_TEXT Version number
     */
    public function get_version() : string
    {
        return '11.0.1'; // addon_version_auto_update d9f826dc4fba783d69108a0c7718d330
    }

    /**
     * Get the minimum required version of the website software needed to use this addon.
     *
     * @return float Minimum required website software version
     */
    public function get_min_cms_version() : float
    {
        return 11.0;
    }

    /**
     * Get the maximum compatible version of the website software to use this addon.
     *
     * @return ?float Maximum compatible website software version (null: no maximum version currently)
     */
    public function get_max_cms_version() : ?float
    {
        return 11.9;
    }

    /**
     * Get the addon category.
     *
     * @return string The category
     */
    public function get_category() : string
    {
        return 'Development';
    }

    /**
     * Get the addon author.
     *
     * @return string The author
     */
    public function get_author() : string
    {
        return 'Chris Graham';
    }

    /**
     * Find other authors.
     *
     * @return array A list of co-authors that should be attributed
     */
    public function get_copyright_attribution() : array
    {
        return [
            'Incorporates MantisBT',
        ];
    }

    /**
     * Get the addon licence (one-line summary only).
     *
     * @return string The licence
     */
    public function get_licence() : string
    {
        return 'Licensed on the same terms as Composr / MantisBT under GPL';
    }

    /**
     * Get the description of the addon.
     *
     * @return string Description of the addon
     */
    public function get_description() : string
    {
        return 'The homesite issue tracker.';
    }

    /**
     * Get a list of tutorials that apply to this addon.
     *
     * @return array List of tutorials
     */
    public function get_applicable_tutorials() : array
    {
        return [];
    }

    /**
     * Get a mapping of dependency types.
     *
     * @return array A structure specifying dependency information
     */
    public function get_dependencies() : array
    {
        return [
            'requires' => [
                'tickets',
                'ecommerce',
                'points',
                'Conversr',
                //'core_all_icons',
                'cms_homesite',
                'PHP 7.4',
            ],
            'recommends' => [
                'stats',
                'MySQL',
                'cms_release_build',
                'composr_tutorials',
            ],
            'conflicts_with' => [],
            'previously_in_addon' => ['composr_homesite_support_credits'],
        ];
    }

    /**
     * Explicitly say which icon should be used.
     *
     * @return URLPATH Icon
     */
    public function get_default_icon() : string
    {
        return 'themes/default/images/icons/admin/component.svg';
    }

    /**
     * Get a list of files that belong to this addon.
     *
     * @return array List of files
     */
    public function get_file_list() : array
    {
        return [
            'data_custom/tracker.php',
            'lang_custom/EN/tracker.ini',
            'site/pages/modules_custom/tracker.php',
            'sources_custom/hooks/systems/addon_registry/cms_homesite_tracker.php',
            'sources_custom/hooks/systems/symbols/MANTIS_TOTAL.php',
            'sources_custom/hooks/systems/symbols/MANTIS_WEEK_ACTIVITY.php',
            'sources_custom/mantis.php',
            'sources_custom/miniblocks/main_mantis_tracker.php',
            'themes/default/css_custom/tracker.css',
            'themes/default/images_custom/icons/tracker/easy.svg',
            'themes/default/images_custom/icons/tracker/hard.svg',
            'themes/default/images_custom/icons/tracker/index.html',
            'themes/default/images_custom/icons/tracker/minus.svg',
            'themes/default/images_custom/icons/tracker/plus.svg',
            'themes/default/images_custom/icons_monochrome/tracker/easy.svg',
            'themes/default/images_custom/icons_monochrome/tracker/hard.svg',
            'themes/default/images_custom/icons_monochrome/tracker/index.html',
            'themes/default/images_custom/icons_monochrome/tracker/minus.svg',
            'themes/default/images_custom/icons_monochrome/tracker/plus.svg',
            'themes/default/javascript_custom/cms_homesite_tracker.js',
            'themes/default/templates_custom/BLOCK_MAIN_MANTIS_TRACKER.tpl',
            'themes/default/templates_custom/MANTIS_TRACKER.tpl',
            'tracker/.github/dependabot.yml',
            'tracker/.imgbotconfig',
            'tracker/KEYS.md',
            'tracker/SECURITY.md',
            'tracker/account_delete.php',
            'tracker/account_manage_columns_page.php',
            'tracker/account_page.php',
            'tracker/account_prefs_inc.php',
            'tracker/account_prefs_page.php',
            'tracker/account_prefs_reset.php',
            'tracker/account_prefs_update.php',
            'tracker/account_prof_edit_page.php',
            'tracker/account_prof_menu_page.php',
            'tracker/account_prof_update.php',
            'tracker/account_sponsor_page.php',
            'tracker/account_sponsor_update.php',
            'tracker/account_update.php',
            'tracker/adm_config_delete.php',
            'tracker/adm_config_page.php',
            'tracker/adm_config_report.php',
            'tracker/adm_config_set.php',
            'tracker/adm_permissions_report.php',
            'tracker/api/rest/.htaccess',
            'tracker/api/rest/api.postman_collection.json',
            'tracker/api/rest/env.postman_environment.json',
            'tracker/api/rest/index.php',
            'tracker/api/rest/restcore/.htaccess',
            'tracker/api/rest/restcore/ApiEnabledMiddleware.php',
            'tracker/api/rest/restcore/AuthMiddleware.php',
            'tracker/api/rest/restcore/CacheMiddleware.php',
            'tracker/api/rest/restcore/OfflineMiddleware.php',
            'tracker/api/rest/restcore/VersionMiddleware.php',
            'tracker/api/rest/restcore/config_rest.php',
            'tracker/api/rest/restcore/filters_rest.php',
            'tracker/api/rest/restcore/internal_rest.php',
            'tracker/api/rest/restcore/issues_rest.php',
            'tracker/api/rest/restcore/lang_rest.php',
            'tracker/api/rest/restcore/pages_rest.php',
            'tracker/api/rest/restcore/projects_rest.php',
            'tracker/api/rest/restcore/users_rest.php',
            'tracker/api/rest/swagger.json',
            'tracker/api/rest/web.config',
            'tracker/api/soap/mantisconnect.php',
            'tracker/api/soap/mantisconnect.wsdl',
            'tracker/api/soap/mc_account_api.php',
            'tracker/api/soap/mc_api.php',
            'tracker/api/soap/mc_config_api.php',
            'tracker/api/soap/mc_core.php',
            'tracker/api/soap/mc_custom_field_api.php',
            'tracker/api/soap/mc_enum_api.php',
            'tracker/api/soap/mc_file_api.php',
            'tracker/api/soap/mc_filter_api.php',
            'tracker/api/soap/mc_issue_api.php',
            'tracker/api/soap/mc_issue_attachment_api.php',
            'tracker/api/soap/mc_project_api.php',
            'tracker/api/soap/mc_project_attachment_api.php',
            'tracker/api/soap/mc_tag_api.php',
            'tracker/api/soap/mc_user_pref_api.php',
            'tracker/api/soap/mc_user_profile_api.php',
            'tracker/api/soap/wsdl-viewer.xsl',
            'tracker/api_token_create.php',
            'tracker/api_token_revoke.php',
            'tracker/api_tokens_page.php',
            'tracker/billing_export_to_csv.php',
            'tracker/billing_export_to_excel.php',
            'tracker/billing_inc.php',
            'tracker/billing_page.php',
            'tracker/browser_search_plugin.php',
            'tracker/bug_actiongroup.php',
            'tracker/bug_actiongroup_add_note_inc.php',
            'tracker/bug_actiongroup_attach_tags_inc.php',
            'tracker/bug_actiongroup_ext.php',
            'tracker/bug_actiongroup_ext_page.php',
            'tracker/bug_actiongroup_page.php',
            'tracker/bug_actiongroup_update_product_build_inc.php',
            'tracker/bug_actiongroup_update_severity_inc.php',
            'tracker/bug_change_status_page.php',
            'tracker/bug_file_delete.php',
            'tracker/bug_monitor_add.php',
            'tracker/bug_monitor_delete.php',
            'tracker/bug_relationship_add.php',
            'tracker/bug_relationship_delete.php',
            'tracker/bug_relationship_graph.php',
            'tracker/bug_relationship_graph_img.php',
            'tracker/bug_reminder.php',
            'tracker/bug_reminder_page.php',
            'tracker/bug_report.php',
            'tracker/bug_report_page.php',
            'tracker/bug_revision_drop.php',
            'tracker/bug_revision_view_page.php',
            'tracker/bug_set_sponsorship.php',
            'tracker/bug_sponsorship_list_view_inc.php',
            'tracker/bug_stick.php',
            'tracker/bug_update.php',
            'tracker/bug_update_page.php',
            'tracker/bug_view_advanced_page.php',
            'tracker/bug_view_inc.php',
            'tracker/bug_view_page.php',
            'tracker/bugnote_add.php',
            'tracker/bugnote_add_inc.php',
            'tracker/bugnote_delete.php',
            'tracker/bugnote_edit_page.php',
            'tracker/bugnote_set_view_state.php',
            'tracker/bugnote_stats_inc.php',
            'tracker/bugnote_update.php',
            'tracker/bugnote_view_inc.php',
            'tracker/build/.htaccess',
            'tracker/build/README.md',
            'tracker/build/Web.config',
            'tracker/build/buildrelease-repo.py',
            'tracker/build/buildrelease.py',
            'tracker/build/check_apostrophe.php',
            'tracker/build/check_formatting.php',
            'tracker/build/check_lineterm.php',
            'tracker/build/docbook-manual-repo.py',
            'tracker/build/docbook-manual.py',
            'tracker/build/travis_before_script.sh',
            'tracker/build/travis_script.sh',
            'tracker/build/update-copyright.sh',
            'tracker/build/update-credits.sh',
            'tracker/build/update-fonts.php',
            'tracker/changelog_page.php',
            'tracker/composer.json',
            'tracker/composer.lock',
            'tracker/config/.htaccess',
            'tracker/config/Web.config',
            'tracker/config/config_inc.php',
            'tracker/config/config_inc.php.sample',
            'tracker/config/custom_constants_inc.php',
            'tracker/config_defaults_inc.php',
            'tracker/core.php',
            'tracker/core/.htaccess',
            'tracker/core/Web.config',
            'tracker/core/access_api.php',
            'tracker/core/antispam_api.php',
            'tracker/core/api_token_api.php',
            'tracker/core/authentication_api.php',
            'tracker/core/billing_api.php',
            'tracker/core/bug_activity_api.php',
            'tracker/core/bug_api.php',
            'tracker/core/bug_group_action_api.php',
            'tracker/core/bug_revision_api.php',
            'tracker/core/bugnote_api.php',
            'tracker/core/category_api.php',
            'tracker/core/cfdefs/cfdef_standard.php',
            'tracker/core/classes/AuthFlags.class.php',
            'tracker/core/classes/Avatar.class.php',
            'tracker/core/classes/BugFilterQuery.class.php',
            'tracker/core/classes/ConfigParser.class.php',
            'tracker/core/classes/DbQuery.class.php',
            'tracker/core/classes/FilterConverter.class.php',
            'tracker/core/classes/InvalidDefinitionPlugin.class.php',
            'tracker/core/classes/InvalidPlugin.class.php',
            'tracker/core/classes/IssueAssignedTimelineEvent.class.php',
            'tracker/core/classes/IssueAttachmentTimelineEvent.class.php',
            'tracker/core/classes/IssueCreatedTimelineEvent.class.php',
            'tracker/core/classes/IssueMonitorTimelineEvent.class.php',
            'tracker/core/classes/IssueNoteCreatedTimelineEvent.class.php',
            'tracker/core/classes/IssueStatusChangeTimelineEvent.class.php',
            'tracker/core/classes/IssueTagTimelineEvent.class.php',
            'tracker/core/classes/MantisColumn.class.php',
            'tracker/core/classes/MantisCorePlugin.class.php',
            'tracker/core/classes/MantisCoreWikiPlugin.class.php',
            'tracker/core/classes/MantisEnum.class.php',
            'tracker/core/classes/MantisFilter.class.php',
            'tracker/core/classes/MantisFormattingPlugin.class.php',
            'tracker/core/classes/MantisPlugin.class.php',
            'tracker/core/classes/MantisWikiPlugin.class.php',
            'tracker/core/classes/MissingClassPlugin.class.php',
            'tracker/core/classes/MissingPlugin.class.php',
            'tracker/core/classes/TimelineEvent.class.php',
            'tracker/core/classes/Tokenizer.class.php',
            'tracker/core/collapse_api.php',
            'tracker/core/columns_api.php',
            'tracker/core/commands/Command.php',
            'tracker/core/commands/ConfigsGetCommand.php',
            'tracker/core/commands/ConfigsSetCommand.php',
            'tracker/core/commands/IssueAddCommand.php',
            'tracker/core/commands/IssueDeleteCommand.php',
            'tracker/core/commands/IssueFileAddCommand.php',
            'tracker/core/commands/IssueFileGetCommand.php',
            'tracker/core/commands/IssueNoteAddCommand.php',
            'tracker/core/commands/IssueNoteDeleteCommand.php',
            'tracker/core/commands/IssueRelationshipAddCommand.php',
            'tracker/core/commands/IssueRelationshipDeleteCommand.php',
            'tracker/core/commands/IssueViewPageCommand.php',
            'tracker/core/commands/LocalizedStringsGetCommand.php',
            'tracker/core/commands/MonitorAddCommand.php',
            'tracker/core/commands/ProjectAddCommand.php',
            'tracker/core/commands/ProjectDeleteCommand.php',
            'tracker/core/commands/ProjectHierarchyAddCommand.php',
            'tracker/core/commands/ProjectHierarchyDeleteCommand.php',
            'tracker/core/commands/ProjectHierarchyUpdateCommand.php',
            'tracker/core/commands/ProjectUpdateCommand.php',
            'tracker/core/commands/ProjectUsersAddCommand.php',
            'tracker/core/commands/ProjectUsersDeleteCommand.php',
            'tracker/core/commands/ProjectUsersGetCommand.php',
            'tracker/core/commands/TagAttachCommand.php',
            'tracker/core/commands/TagDetachCommand.php',
            'tracker/core/commands/UserCreateCommand.php',
            'tracker/core/commands/UserDeleteCommand.php',
            'tracker/core/commands/UserGetCommand.php',
            'tracker/core/commands/UserResetPasswordCommand.php',
            'tracker/core/commands/UserTokenCreateCommand.php',
            'tracker/core/commands/UserTokenDeleteCommand.php',
            'tracker/core/commands/UserUpdateCommand.php',
            'tracker/core/commands/VersionAddCommand.php',
            'tracker/core/commands/VersionDeleteCommand.php',
            'tracker/core/commands/VersionGetCommand.php',
            'tracker/core/commands/VersionUpdateCommand.php',
            'tracker/core/compress_api.php',
            'tracker/core/config_api.php',
            'tracker/core/constant_inc.php',
            'tracker/core/crypto_api.php',
            'tracker/core/csv_api.php',
            'tracker/core/current_user_api.php',
            'tracker/core/custom_field_api.php',
            'tracker/core/custom_function_api.php',
            'tracker/core/database_api.php',
            'tracker/core/date_api.php',
            'tracker/core/email_api.php',
            'tracker/core/email_queue_api.php',
            'tracker/core/error_api.php',
            'tracker/core/event_api.php',
            'tracker/core/events_inc.php',
            'tracker/core/excel_api.php',
            'tracker/core/exceptions/ClientException.php',
            'tracker/core/exceptions/LegacyApiFaultException.php',
            'tracker/core/exceptions/MantisException.php',
            'tracker/core/exceptions/ServiceException.php',
            'tracker/core/exceptions/StateException.php',
            'tracker/core/file_api.php',
            'tracker/core/filter_api.php',
            'tracker/core/filter_constants_inc.php',
            'tracker/core/filter_form_api.php',
            'tracker/core/form_api.php',
            'tracker/core/gpc_api.php',
            'tracker/core/graphviz_api.php',
            'tracker/core/helper_api.php',
            'tracker/core/history_api.php',
            'tracker/core/html_api.php',
            'tracker/core/http_api.php',
            'tracker/core/icon_api.php',
            'tracker/core/install_helper_functions_api.php',
            'tracker/core/json_api.php',
            'tracker/core/lang_api.php',
            'tracker/core/last_visited_api.php',
            'tracker/core/layout_api.php',
            'tracker/core/ldap_api.php',
            'tracker/core/logging_api.php',
            'tracker/core/mention_api.php',
            'tracker/core/news_api.php',
            'tracker/core/obsolete.php',
            'tracker/core/php_api.php',
            'tracker/core/plugin_api.php',
            'tracker/core/prepare_api.php',
            'tracker/core/print_api.php',
            'tracker/core/profile_api.php',
            'tracker/core/project_api.php',
            'tracker/core/project_hierarchy_api.php',
            'tracker/core/relationship_api.php',
            'tracker/core/relationship_graph_api.php',
            'tracker/core/rss_api.php',
            'tracker/core/session_api.php',
            'tracker/core/sponsorship_api.php',
            'tracker/core/string_api.php',
            'tracker/core/summary_api.php',
            'tracker/core/tag_api.php',
            'tracker/core/timeline_api.php',
            'tracker/core/timeline_inc.php',
            'tracker/core/tokens_api.php',
            'tracker/core/url_api.php',
            'tracker/core/user_api.php',
            'tracker/core/user_pref_api.php',
            'tracker/core/utility_api.php',
            'tracker/core/version_api.php',
            'tracker/core/wiki_api.php',
            'tracker/core/workflow_api.php',
            'tracker/css/ace-mantis.css',
            'tracker/css/ace-part2.css',
            'tracker/css/ace-rtl.css',
            'tracker/css/ace-rtl.min.css',
            'tracker/css/ace-skins.css',
            'tracker/css/ace-skins.min.css',
            'tracker/css/ace.css',
            'tracker/css/ace.min.css',
            'tracker/css/bootstrap-3.4.1.css',
            'tracker/css/bootstrap-3.4.1.min.css',
            'tracker/css/bootstrap-datetimepicker-4.17.47.min.css',
            'tracker/css/default.css',
            'tracker/css/dropzone-5.5.0.css',
            'tracker/css/dropzone-5.5.0.min.css',
            'tracker/css/font-awesome-4.7.0.min.css',
            'tracker/css/fonts.css',
            'tracker/css/images/ajax-loader.gif',
            'tracker/css/images/ajax-loader.png',
            'tracker/css/images/icons-18-black.png',
            'tracker/css/images/icons-18-white.png',
            'tracker/css/images/icons-36-black.png',
            'tracker/css/images/icons-36-white.png',
            'tracker/css/images/pattern.jpg',
            'tracker/css/images/ui-bg_diagonals-thick_18_b81900_40x40.png',
            'tracker/css/images/ui-bg_diagonals-thick_20_666666_40x40.png',
            'tracker/css/images/ui-bg_flat_0_aaaaaa_40x100.png',
            'tracker/css/images/ui-bg_flat_10_000000_40x100.png',
            'tracker/css/images/ui-bg_flat_75_ffffff_40x100.png',
            'tracker/css/images/ui-bg_glass_100_f6f6f6_1x400.png',
            'tracker/css/images/ui-bg_glass_100_fdf5ce_1x400.png',
            'tracker/css/images/ui-bg_glass_55_fbf9ee_1x400.png',
            'tracker/css/images/ui-bg_glass_65_ffffff_1x400.png',
            'tracker/css/images/ui-bg_glass_75_dadada_1x400.png',
            'tracker/css/images/ui-bg_glass_75_e6e6e6_1x400.png',
            'tracker/css/images/ui-bg_glass_95_fef1ec_1x400.png',
            'tracker/css/images/ui-bg_gloss-wave_35_f6a828_500x100.png',
            'tracker/css/images/ui-bg_highlight-soft_100_eeeeee_1x100.png',
            'tracker/css/images/ui-bg_highlight-soft_75_cccccc_1x100.png',
            'tracker/css/images/ui-bg_highlight-soft_75_ffe45c_1x100.png',
            'tracker/css/images/ui-icons_222222_256x240.png',
            'tracker/css/images/ui-icons_228ef1_256x240.png',
            'tracker/css/images/ui-icons_2e83ff_256x240.png',
            'tracker/css/images/ui-icons_454545_256x240.png',
            'tracker/css/images/ui-icons_888888_256x240.png',
            'tracker/css/images/ui-icons_cd0a0a_256x240.png',
            'tracker/css/images/ui-icons_ef8c08_256x240.png',
            'tracker/css/images/ui-icons_ffd27a_256x240.png',
            'tracker/css/images/ui-icons_ffffff_256x240.png',
            'tracker/css/login.css',
            'tracker/css/rtl.css',
            'tracker/css/status_config.php',
            'tracker/csv_export.php',
            'tracker/excel_xml_export.php',
            'tracker/file_download.php',
            'tracker/fonts/.htaccess',
            'tracker/fonts/fontawesome-webfont.eot',
            'tracker/fonts/fontawesome-webfont.svg',
            'tracker/fonts/fontawesome-webfont.ttf',
            'tracker/fonts/fontawesome-webfont.woff',
            'tracker/fonts/fontawesome-webfont.woff2',
            'tracker/fonts/glyphicons-halflings-regular.eot',
            'tracker/fonts/glyphicons-halflings-regular.svg',
            'tracker/fonts/glyphicons-halflings-regular.ttf',
            'tracker/fonts/glyphicons-halflings-regular.woff',
            'tracker/fonts/glyphicons-halflings-regular.woff2',
            'tracker/fonts/montserrat-v25-cyrillic-ext-regular.woff',
            'tracker/fonts/montserrat-v25-cyrillic-ext-regular.woff2',
            'tracker/fonts/montserrat-v25-cyrillic-regular.woff',
            'tracker/fonts/montserrat-v25-cyrillic-regular.woff2',
            'tracker/fonts/montserrat-v25-latin-ext-regular.woff',
            'tracker/fonts/montserrat-v25-latin-ext-regular.woff2',
            'tracker/fonts/montserrat-v25-latin-regular.woff',
            'tracker/fonts/montserrat-v25-latin-regular.woff2',
            'tracker/fonts/montserrat-v25-vietnamese-regular.woff',
            'tracker/fonts/montserrat-v25-vietnamese-regular.woff2',
            'tracker/fonts/open-sans-v35-cyrillic-ext-regular.woff',
            'tracker/fonts/open-sans-v35-cyrillic-ext-regular.woff2',
            'tracker/fonts/open-sans-v35-cyrillic-regular.woff',
            'tracker/fonts/open-sans-v35-cyrillic-regular.woff2',
            'tracker/fonts/open-sans-v35-greek-ext-regular.woff',
            'tracker/fonts/open-sans-v35-greek-ext-regular.woff2',
            'tracker/fonts/open-sans-v35-greek-regular.woff',
            'tracker/fonts/open-sans-v35-greek-regular.woff2',
            'tracker/fonts/open-sans-v35-hebrew-regular.woff',
            'tracker/fonts/open-sans-v35-hebrew-regular.woff2',
            'tracker/fonts/open-sans-v35-latin-ext-regular.woff',
            'tracker/fonts/open-sans-v35-latin-ext-regular.woff2',
            'tracker/fonts/open-sans-v35-latin-regular.woff',
            'tracker/fonts/open-sans-v35-latin-regular.woff2',
            'tracker/fonts/open-sans-v35-vietnamese-regular.woff',
            'tracker/fonts/open-sans-v35-vietnamese-regular.woff2',
            'tracker/fonts/poppins-v20-devanagari-regular.woff',
            'tracker/fonts/poppins-v20-devanagari-regular.woff2',
            'tracker/fonts/poppins-v20-latin-ext-regular.woff',
            'tracker/fonts/poppins-v20-latin-ext-regular.woff2',
            'tracker/fonts/poppins-v20-latin-regular.woff',
            'tracker/fonts/poppins-v20-latin-regular.woff2',
            'tracker/images/avatar.png',
            'tracker/images/favicon.ico',
            'tracker/images/mantis_logo.gif',
            'tracker/images/mantis_logo.png',
            'tracker/images/mantis_logo_notext.png',
            'tracker/images/rel_dependant.png',
            'tracker/images/rel_duplicate.png',
            'tracker/images/rel_related.png',
            'tracker/index.php',
            'tracker/issues_rss.php',
            'tracker/javascript_config.php',
            'tracker/javascript_translations.php',
            'tracker/js/ace.js',
            'tracker/js/ace.min.js',
            'tracker/js/adm_config_report.js',
            'tracker/js/bootstrap-3.4.1.js',
            'tracker/js/bootstrap-3.4.1.min.js',
            'tracker/js/bootstrap-datetimepicker-4.17.47.min.js',
            'tracker/js/bugFilter.js',
            'tracker/js/common.js',
            'tracker/js/dropzone-5.5.0.js',
            'tracker/js/dropzone-5.5.0.min.js',
            'tracker/js/install.js',
            'tracker/js/jquery-2.2.4.min.js',
            'tracker/js/list-2.3.1.js',
            'tracker/js/list-2.3.1.min.js',
            'tracker/js/login.js',
            'tracker/js/manage_custom_field_edit_page.js',
            'tracker/js/manage_proj_edit_page.js',
            'tracker/js/moment-with-locales-2.29.4.min.js',
            'tracker/js/typeahead.jquery-1.3.4.min.js',
            'tracker/jump_to_bug.php',
            'tracker/lang/.htaccess',
            'tracker/lang/README',
            'tracker/lang/Web.config',
            'tracker/lang/strings_afrikaans.txt',
            'tracker/lang/strings_amharic.txt',
            'tracker/lang/strings_arabic.txt',
            'tracker/lang/strings_arabicegyptianspoken.txt',
            'tracker/lang/strings_asturian.txt',
            'tracker/lang/strings_basque.txt',
            'tracker/lang/strings_belarusian_tarask.txt',
            'tracker/lang/strings_breton.txt',
            'tracker/lang/strings_bulgarian.txt',
            'tracker/lang/strings_catalan.txt',
            'tracker/lang/strings_chinese_simplified.txt',
            'tracker/lang/strings_chinese_traditional.txt',
            'tracker/lang/strings_croatian.txt',
            'tracker/lang/strings_czech.txt',
            'tracker/lang/strings_danish.txt',
            'tracker/lang/strings_dutch.txt',
            'tracker/lang/strings_english.txt',
            'tracker/lang/strings_esperanto.txt',
            'tracker/lang/strings_estonian.txt',
            'tracker/lang/strings_finnish.txt',
            'tracker/lang/strings_french.txt',
            'tracker/lang/strings_galician.txt',
            'tracker/lang/strings_georgian.txt',
            'tracker/lang/strings_german.txt',
            'tracker/lang/strings_greek.txt',
            'tracker/lang/strings_hebrew.txt',
            'tracker/lang/strings_hindi.txt',
            'tracker/lang/strings_hungarian.txt',
            'tracker/lang/strings_icelandic.txt',
            'tracker/lang/strings_interlingua.txt',
            'tracker/lang/strings_italian.txt',
            'tracker/lang/strings_japanese.txt',
            'tracker/lang/strings_korean.txt',
            'tracker/lang/strings_latvian.txt',
            'tracker/lang/strings_lithuanian.txt',
            'tracker/lang/strings_luxembourgish.txt',
            'tracker/lang/strings_macedonian.txt',
            'tracker/lang/strings_norwegian_bokmal.txt',
            'tracker/lang/strings_norwegian_nynorsk.txt',
            'tracker/lang/strings_occitan.txt',
            'tracker/lang/strings_persian.txt',
            'tracker/lang/strings_polish.txt',
            'tracker/lang/strings_portuguese_brazil.txt',
            'tracker/lang/strings_portuguese_standard.txt',
            'tracker/lang/strings_qqq.txt',
            'tracker/lang/strings_ripoarisch.txt',
            'tracker/lang/strings_romanian.txt',
            'tracker/lang/strings_russian.txt',
            'tracker/lang/strings_saraiki.txt',
            'tracker/lang/strings_serbian.txt',
            'tracker/lang/strings_serbian_latin.txt',
            'tracker/lang/strings_slovak.txt',
            'tracker/lang/strings_slovene.txt',
            'tracker/lang/strings_spanish.txt',
            'tracker/lang/strings_swedish.txt',
            'tracker/lang/strings_swissgerman.txt',
            'tracker/lang/strings_tagalog.txt',
            'tracker/lang/strings_turkish.txt',
            'tracker/lang/strings_ukrainian.txt',
            'tracker/lang/strings_urdu.txt',
            'tracker/lang/strings_vietnamese.txt',
            'tracker/lang/strings_volapuk.txt',
            'tracker/lang/strings_zazaki.txt',
            'tracker/library/.htaccess',
            'tracker/library/README.md',
            'tracker/library/Web.config',
            'tracker/library/rssbuilder/class.ObjectIterator.inc.php',
            'tracker/library/rssbuilder/class.ObjectList.inc.php',
            'tracker/library/rssbuilder/class.RSSBase.inc.php',
            'tracker/library/rssbuilder/class.RSSBuilder.inc.php',
            'tracker/library/rssbuilder/class.RSSItem.inc.php',
            'tracker/library/rssbuilder/class.RSSItemList.inc.php',
            'tracker/library/rssbuilder/class.RSS_V_091.inc.php',
            'tracker/library/rssbuilder/class.RSS_V_100.inc.php',
            'tracker/library/rssbuilder/class.RSS_V_200.inc.php',
            'tracker/library/rssbuilder/class.RSS_V_abstract.inc.php',
            'tracker/library/rssbuilder/doc/changelog_rssbuilder.htm',
            'tracker/library/rssbuilder/doc/doc.css',
            'tracker/library/rssbuilder/index.html',
            'tracker/library/rssbuilder/interface.RSS.inc.php',
            'tracker/library/rssbuilder/readme_mantis.txt',
            'tracker/library/rssbuilder/rss_sample_script.php',
            'tracker/login.php',
            'tracker/login_anon.php',
            'tracker/login_cookie_test.php',
            'tracker/login_page.php',
            'tracker/login_password_page.php',
            'tracker/login_select_proj_page.php',
            'tracker/logout_page.php',
            'tracker/lost_pwd.php',
            'tracker/lost_pwd_page.php',
            'tracker/main_page.php',
            'tracker/manage_columns_copy.php',
            'tracker/manage_columns_inc.php',
            'tracker/manage_config_columns_page.php',
            'tracker/manage_config_columns_reset.php',
            'tracker/manage_config_columns_set.php',
            'tracker/manage_config_email_page.php',
            'tracker/manage_config_email_set.php',
            'tracker/manage_config_revert.php',
            'tracker/manage_config_work_threshold_page.php',
            'tracker/manage_config_work_threshold_set.php',
            'tracker/manage_config_workflow_graph_page.php',
            'tracker/manage_config_workflow_page.php',
            'tracker/manage_config_workflow_set.php',
            'tracker/manage_custom_field_create.php',
            'tracker/manage_custom_field_delete.php',
            'tracker/manage_custom_field_edit_page.php',
            'tracker/manage_custom_field_page.php',
            'tracker/manage_custom_field_proj_add.php',
            'tracker/manage_custom_field_update.php',
            'tracker/manage_filter_delete.php',
            'tracker/manage_filter_edit_page.php',
            'tracker/manage_filter_edit_update.php',
            'tracker/manage_filter_page.php',
            'tracker/manage_overview_page.php',
            'tracker/manage_plugin_install.php',
            'tracker/manage_plugin_page.php',
            'tracker/manage_plugin_uninstall.php',
            'tracker/manage_plugin_update.php',
            'tracker/manage_plugin_upgrade.php',
            'tracker/manage_prof_menu_page.php',
            'tracker/manage_proj_cat_add.php',
            'tracker/manage_proj_cat_copy.php',
            'tracker/manage_proj_cat_delete.php',
            'tracker/manage_proj_cat_edit_page.php',
            'tracker/manage_proj_cat_update.php',
            'tracker/manage_proj_create.php',
            'tracker/manage_proj_create_page.php',
            'tracker/manage_proj_custom_field_add_existing.php',
            'tracker/manage_proj_custom_field_copy.php',
            'tracker/manage_proj_custom_field_remove.php',
            'tracker/manage_proj_custom_field_update.php',
            'tracker/manage_proj_delete.php',
            'tracker/manage_proj_edit_page.php',
            'tracker/manage_proj_page.php',
            'tracker/manage_proj_subproj_add.php',
            'tracker/manage_proj_subproj_delete.php',
            'tracker/manage_proj_update.php',
            'tracker/manage_proj_update_children.php',
            'tracker/manage_proj_user_add.php',
            'tracker/manage_proj_user_copy.php',
            'tracker/manage_proj_user_remove.php',
            'tracker/manage_proj_user_update.php',
            'tracker/manage_proj_ver_add.php',
            'tracker/manage_proj_ver_copy.php',
            'tracker/manage_proj_ver_delete.php',
            'tracker/manage_proj_ver_edit_page.php',
            'tracker/manage_proj_ver_update.php',
            'tracker/manage_tags_page.php',
            'tracker/manage_user_create.php',
            'tracker/manage_user_create_page.php',
            'tracker/manage_user_delete.php',
            'tracker/manage_user_edit_page.php',
            'tracker/manage_user_impersonate.php',
            'tracker/manage_user_page.php',
            'tracker/manage_user_proj_add.php',
            'tracker/manage_user_proj_delete.php',
            'tracker/manage_user_prune.php',
            'tracker/manage_user_reset.php',
            'tracker/manage_user_update.php',
            'tracker/mantis_offline.php.sample',
            'tracker/my_view_inc.php',
            'tracker/my_view_page.php',
            'tracker/news_add.php',
            'tracker/news_edit_page.php',
            'tracker/news_list_page.php',
            'tracker/news_menu_page.php',
            'tracker/news_rss.php',
            'tracker/news_update.php',
            'tracker/news_view_page.php',
            'tracker/permalink_page.php',
            'tracker/phpunit.xml',
            'tracker/plugin.php',
            'tracker/plugin_file.php',
            'tracker/plugins/.htaccess',
            'tracker/plugins/Composr/Composr.php',
            'tracker/plugins/Composr/lang/strings_english.txt',
            'tracker/plugins/Gravatar/Gravatar.php',
            'tracker/plugins/Gravatar/README.md',
            'tracker/plugins/Gravatar/lang/strings_arabic.txt',
            'tracker/plugins/Gravatar/lang/strings_asturian.txt',
            'tracker/plugins/Gravatar/lang/strings_belarusian_tarask.txt',
            'tracker/plugins/Gravatar/lang/strings_breton.txt',
            'tracker/plugins/Gravatar/lang/strings_bulgarian.txt',
            'tracker/plugins/Gravatar/lang/strings_chinese_simplified.txt',
            'tracker/plugins/Gravatar/lang/strings_chinese_traditional.txt',
            'tracker/plugins/Gravatar/lang/strings_czech.txt',
            'tracker/plugins/Gravatar/lang/strings_danish.txt',
            'tracker/plugins/Gravatar/lang/strings_dutch.txt',
            'tracker/plugins/Gravatar/lang/strings_en-gb.txt',
            'tracker/plugins/Gravatar/lang/strings_english.txt',
            'tracker/plugins/Gravatar/lang/strings_esperanto.txt',
            'tracker/plugins/Gravatar/lang/strings_french.txt',
            'tracker/plugins/Gravatar/lang/strings_galician.txt',
            'tracker/plugins/Gravatar/lang/strings_german.txt',
            'tracker/plugins/Gravatar/lang/strings_greek.txt',
            'tracker/plugins/Gravatar/lang/strings_hebrew.txt',
            'tracker/plugins/Gravatar/lang/strings_hindi.txt',
            'tracker/plugins/Gravatar/lang/strings_hungarian.txt',
            'tracker/plugins/Gravatar/lang/strings_icelandic.txt',
            'tracker/plugins/Gravatar/lang/strings_indonesian.txt',
            'tracker/plugins/Gravatar/lang/strings_interlingua.txt',
            'tracker/plugins/Gravatar/lang/strings_italian.txt',
            'tracker/plugins/Gravatar/lang/strings_japanese.txt',
            'tracker/plugins/Gravatar/lang/strings_korean.txt',
            'tracker/plugins/Gravatar/lang/strings_lithuanian.txt',
            'tracker/plugins/Gravatar/lang/strings_macedonian.txt',
            'tracker/plugins/Gravatar/lang/strings_norwegian_bokmal.txt',
            'tracker/plugins/Gravatar/lang/strings_persian.txt',
            'tracker/plugins/Gravatar/lang/strings_polish.txt',
            'tracker/plugins/Gravatar/lang/strings_portuguese_brazil.txt',
            'tracker/plugins/Gravatar/lang/strings_portuguese_standard.txt',
            'tracker/plugins/Gravatar/lang/strings_russian.txt',
            'tracker/plugins/Gravatar/lang/strings_saraiki.txt',
            'tracker/plugins/Gravatar/lang/strings_serbian_latin.txt',
            'tracker/plugins/Gravatar/lang/strings_slovak.txt',
            'tracker/plugins/Gravatar/lang/strings_slovene.txt',
            'tracker/plugins/Gravatar/lang/strings_spanish.txt',
            'tracker/plugins/Gravatar/lang/strings_swedish.txt',
            'tracker/plugins/Gravatar/lang/strings_turkish.txt',
            'tracker/plugins/Gravatar/lang/strings_ukrainian.txt',
            'tracker/plugins/Gravatar/lang/strings_vietnamese.txt',
            'tracker/plugins/Gravatar/lang/strings_zazaki.txt',
            'tracker/plugins/MantisCoreFormatting/MantisCoreFormatting.php',
            'tracker/plugins/MantisCoreFormatting/core/MantisMarkdown.php',
            'tracker/plugins/MantisCoreFormatting/files/markdown.css',
            'tracker/plugins/MantisCoreFormatting/lang/strings_afrikaans.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_arabic.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_asturian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_basque.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_belarusian_tarask.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_bengali.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_bosnian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_breton.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_bulgarian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_catalan.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_chinese_simplified.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_chinese_traditional.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_czech.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_danish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_dutch.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_english.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_esperanto.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_finnish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_french.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_galician.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_georgian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_german.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_greek.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_hebrew.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_hindi.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_hungarian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_icelandic.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_indonesian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_interlingua.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_italian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_japanese.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_korean.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_laki.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_lezgi.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_lithuanian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_lower_sorbian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_luxembourgish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_macedonian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_malay.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_malayalam.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_norwegian_bokmal.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_occitan.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_persian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_piedmontese.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_polish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_portuguese_brazil.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_portuguese_standard.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_qqq.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_ripoarisch.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_romanian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_russian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_serbian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_serbian_latin.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_slovak.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_slovene.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_spanish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_swedish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_swissgerman.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_tachelhit.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_tagalog.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_telugu.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_turkish.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_ukrainian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_upper_sorbian.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_vietnamese.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_zazaki.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_zh-cn.txt',
            'tracker/plugins/MantisCoreFormatting/lang/strings_zh-hk.txt',
            'tracker/plugins/MantisCoreFormatting/pages/config.php',
            'tracker/plugins/MantisCoreFormatting/pages/config_edit.php',
            'tracker/plugins/MantisCoreFormatting/tests/MarkdownTest.php',
            'tracker/plugins/MantisGraph/MantisGraph.php',
            'tracker/plugins/MantisGraph/core/Period.php',
            'tracker/plugins/MantisGraph/core/graph_api.php',
            'tracker/plugins/MantisGraph/files/Chart-2.9.4.min.js',
            'tracker/plugins/MantisGraph/files/MantisGraph.js',
            'tracker/plugins/MantisGraph/files/chartjs-plugin-colorschemes-0.4.0.min.js',
            'tracker/plugins/MantisGraph/lang/strings_afrikaans.txt',
            'tracker/plugins/MantisGraph/lang/strings_arabic.txt',
            'tracker/plugins/MantisGraph/lang/strings_arabicegyptianspoken.txt',
            'tracker/plugins/MantisGraph/lang/strings_asturian.txt',
            'tracker/plugins/MantisGraph/lang/strings_basque.txt',
            'tracker/plugins/MantisGraph/lang/strings_belarusian_tarask.txt',
            'tracker/plugins/MantisGraph/lang/strings_breton.txt',
            'tracker/plugins/MantisGraph/lang/strings_bulgarian.txt',
            'tracker/plugins/MantisGraph/lang/strings_catalan.txt',
            'tracker/plugins/MantisGraph/lang/strings_chinese_simplified.txt',
            'tracker/plugins/MantisGraph/lang/strings_chinese_traditional.txt',
            'tracker/plugins/MantisGraph/lang/strings_czech.txt',
            'tracker/plugins/MantisGraph/lang/strings_danish.txt',
            'tracker/plugins/MantisGraph/lang/strings_dutch.txt',
            'tracker/plugins/MantisGraph/lang/strings_english.txt',
            'tracker/plugins/MantisGraph/lang/strings_esperanto.txt',
            'tracker/plugins/MantisGraph/lang/strings_estonian.txt',
            'tracker/plugins/MantisGraph/lang/strings_finnish.txt',
            'tracker/plugins/MantisGraph/lang/strings_french.txt',
            'tracker/plugins/MantisGraph/lang/strings_galician.txt',
            'tracker/plugins/MantisGraph/lang/strings_georgian.txt',
            'tracker/plugins/MantisGraph/lang/strings_german.txt',
            'tracker/plugins/MantisGraph/lang/strings_greek.txt',
            'tracker/plugins/MantisGraph/lang/strings_hebrew.txt',
            'tracker/plugins/MantisGraph/lang/strings_hindi.txt',
            'tracker/plugins/MantisGraph/lang/strings_hungarian.txt',
            'tracker/plugins/MantisGraph/lang/strings_icelandic.txt',
            'tracker/plugins/MantisGraph/lang/strings_interlingua.txt',
            'tracker/plugins/MantisGraph/lang/strings_italian.txt',
            'tracker/plugins/MantisGraph/lang/strings_japanese.txt',
            'tracker/plugins/MantisGraph/lang/strings_korean.txt',
            'tracker/plugins/MantisGraph/lang/strings_laki.txt',
            'tracker/plugins/MantisGraph/lang/strings_lithuanian.txt',
            'tracker/plugins/MantisGraph/lang/strings_luxembourgish.txt',
            'tracker/plugins/MantisGraph/lang/strings_macedonian.txt',
            'tracker/plugins/MantisGraph/lang/strings_norwegian_bokmal.txt',
            'tracker/plugins/MantisGraph/lang/strings_norwegian_nynorsk.txt',
            'tracker/plugins/MantisGraph/lang/strings_occitan.txt',
            'tracker/plugins/MantisGraph/lang/strings_persian.txt',
            'tracker/plugins/MantisGraph/lang/strings_piedmontese.txt',
            'tracker/plugins/MantisGraph/lang/strings_polish.txt',
            'tracker/plugins/MantisGraph/lang/strings_portuguese_brazil.txt',
            'tracker/plugins/MantisGraph/lang/strings_portuguese_standard.txt',
            'tracker/plugins/MantisGraph/lang/strings_qqq.txt',
            'tracker/plugins/MantisGraph/lang/strings_ripoarisch.txt',
            'tracker/plugins/MantisGraph/lang/strings_romanian.txt',
            'tracker/plugins/MantisGraph/lang/strings_russian.txt',
            'tracker/plugins/MantisGraph/lang/strings_saraiki.txt',
            'tracker/plugins/MantisGraph/lang/strings_scn.txt',
            'tracker/plugins/MantisGraph/lang/strings_serbian.txt',
            'tracker/plugins/MantisGraph/lang/strings_serbian_latin.txt',
            'tracker/plugins/MantisGraph/lang/strings_slovak.txt',
            'tracker/plugins/MantisGraph/lang/strings_slovene.txt',
            'tracker/plugins/MantisGraph/lang/strings_spanish.txt',
            'tracker/plugins/MantisGraph/lang/strings_swedish.txt',
            'tracker/plugins/MantisGraph/lang/strings_swissgerman.txt',
            'tracker/plugins/MantisGraph/lang/strings_tagalog.txt',
            'tracker/plugins/MantisGraph/lang/strings_turkish.txt',
            'tracker/plugins/MantisGraph/lang/strings_ukrainian.txt',
            'tracker/plugins/MantisGraph/lang/strings_urdu.txt',
            'tracker/plugins/MantisGraph/lang/strings_vietnamese.txt',
            'tracker/plugins/MantisGraph/lang/strings_volapuk.txt',
            'tracker/plugins/MantisGraph/lang/strings_zazaki.txt',
            'tracker/plugins/MantisGraph/lang/strings_zh-cn.txt',
            'tracker/plugins/MantisGraph/pages/category_graph.php',
            'tracker/plugins/MantisGraph/pages/developer_graph.php',
            'tracker/plugins/MantisGraph/pages/issues_trend_bycategory_table.php',
            'tracker/plugins/MantisGraph/pages/issues_trend_bystatus_table.php',
            'tracker/plugins/MantisGraph/pages/issues_trend_graph.php',
            'tracker/plugins/MantisGraph/pages/issues_trend_page.php',
            'tracker/plugins/MantisGraph/pages/priority_graph.php',
            'tracker/plugins/MantisGraph/pages/reporter_graph.php',
            'tracker/plugins/MantisGraph/pages/resolution_graph.php',
            'tracker/plugins/MantisGraph/pages/severity_graph.php',
            'tracker/plugins/MantisGraph/pages/status_graph.php',
            'tracker/plugins/Web.config',
            'tracker/plugins/XmlImportExport/ImportXml.php',
            'tracker/plugins/XmlImportExport/ImportXml/Interface.php',
            'tracker/plugins/XmlImportExport/ImportXml/Issue.php',
            'tracker/plugins/XmlImportExport/ImportXml/Mapper.php',
            'tracker/plugins/XmlImportExport/XmlImportExport.php',
            'tracker/plugins/XmlImportExport/lang/strings_afrikaans.txt',
            'tracker/plugins/XmlImportExport/lang/strings_arabic.txt',
            'tracker/plugins/XmlImportExport/lang/strings_arabicegyptianspoken.txt',
            'tracker/plugins/XmlImportExport/lang/strings_asturian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_belarusian_tarask.txt',
            'tracker/plugins/XmlImportExport/lang/strings_bengali.txt',
            'tracker/plugins/XmlImportExport/lang/strings_breton.txt',
            'tracker/plugins/XmlImportExport/lang/strings_bulgarian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_catalan.txt',
            'tracker/plugins/XmlImportExport/lang/strings_chinese_simplified.txt',
            'tracker/plugins/XmlImportExport/lang/strings_chinese_traditional.txt',
            'tracker/plugins/XmlImportExport/lang/strings_czech.txt',
            'tracker/plugins/XmlImportExport/lang/strings_danish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_dutch.txt',
            'tracker/plugins/XmlImportExport/lang/strings_english.txt',
            'tracker/plugins/XmlImportExport/lang/strings_esperanto.txt',
            'tracker/plugins/XmlImportExport/lang/strings_finnish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_french.txt',
            'tracker/plugins/XmlImportExport/lang/strings_galician.txt',
            'tracker/plugins/XmlImportExport/lang/strings_georgian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_german.txt',
            'tracker/plugins/XmlImportExport/lang/strings_greek.txt',
            'tracker/plugins/XmlImportExport/lang/strings_hebrew.txt',
            'tracker/plugins/XmlImportExport/lang/strings_hindi.txt',
            'tracker/plugins/XmlImportExport/lang/strings_hungarian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_icelandic.txt',
            'tracker/plugins/XmlImportExport/lang/strings_indonesian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_interlingua.txt',
            'tracker/plugins/XmlImportExport/lang/strings_italian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_japanese.txt',
            'tracker/plugins/XmlImportExport/lang/strings_korean.txt',
            'tracker/plugins/XmlImportExport/lang/strings_lithuanian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_luxembourgish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_macedonian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_mirandese.txt',
            'tracker/plugins/XmlImportExport/lang/strings_norwegian_bokmal.txt',
            'tracker/plugins/XmlImportExport/lang/strings_occitan.txt',
            'tracker/plugins/XmlImportExport/lang/strings_piedmontese.txt',
            'tracker/plugins/XmlImportExport/lang/strings_polish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_portuguese_brazil.txt',
            'tracker/plugins/XmlImportExport/lang/strings_portuguese_standard.txt',
            'tracker/plugins/XmlImportExport/lang/strings_qqq.txt',
            'tracker/plugins/XmlImportExport/lang/strings_ripoarisch.txt',
            'tracker/plugins/XmlImportExport/lang/strings_romanian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_russian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_serbian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_serbian_latin.txt',
            'tracker/plugins/XmlImportExport/lang/strings_slovak.txt',
            'tracker/plugins/XmlImportExport/lang/strings_slovene.txt',
            'tracker/plugins/XmlImportExport/lang/strings_spanish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_swedish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_swissgerman.txt',
            'tracker/plugins/XmlImportExport/lang/strings_tagalog.txt',
            'tracker/plugins/XmlImportExport/lang/strings_tunisian_arabic.txt',
            'tracker/plugins/XmlImportExport/lang/strings_turkish.txt',
            'tracker/plugins/XmlImportExport/lang/strings_ukrainian.txt',
            'tracker/plugins/XmlImportExport/lang/strings_vietnamese.txt',
            'tracker/plugins/XmlImportExport/lang/strings_zazaki.txt',
            'tracker/plugins/XmlImportExport/lang/strings_zh-cn.txt',
            'tracker/plugins/XmlImportExport/lang/strings_zh-hk.txt',
            'tracker/plugins/XmlImportExport/mantis.dtd',
            'tracker/plugins/XmlImportExport/pages/config.php',
            'tracker/plugins/XmlImportExport/pages/config_page.php',
            'tracker/plugins/XmlImportExport/pages/export.php',
            'tracker/plugins/XmlImportExport/pages/import.php',
            'tracker/plugins/XmlImportExport/pages/import_action.php',
            'tracker/print_all_bug_options_inc.php',
            'tracker/print_all_bug_options_page.php',
            'tracker/print_all_bug_options_reset.php',
            'tracker/print_all_bug_options_update.php',
            'tracker/print_all_bug_page.php',
            'tracker/print_all_bug_page_word.php',
            'tracker/proj_doc_add.php',
            'tracker/proj_doc_add_page.php',
            'tracker/proj_doc_delete.php',
            'tracker/proj_doc_edit_page.php',
            'tracker/proj_doc_page.php',
            'tracker/proj_doc_update.php',
            'tracker/query_store.php',
            'tracker/query_store_page.php',
            'tracker/readme.md',
            'tracker/return_dynamic_filters.php',
            'tracker/roadmap_page.php',
            'tracker/scripts/.htaccess',
            'tracker/scripts/README.md',
            'tracker/scripts/Web.config',
            'tracker/scripts/cronjob.php',
            'tracker/scripts/send_emails.php',
            'tracker/search.php',
            'tracker/set_project.php',
            'tracker/signup.php',
            'tracker/signup_page.php',
            'tracker/summary_page.php',
            'tracker/tag_attach.php',
            'tracker/tag_create.php',
            'tracker/tag_delete.php',
            'tracker/tag_detach.php',
            'tracker/tag_update.php',
            'tracker/tag_update_page.php',
            'tracker/tag_view_page.php',
            'tracker/uploads/index.html',
            'tracker/vendor/.htaccess',
            'tracker/vendor/Web.config',
            'tracker/vendor/adodb/adodb-php/.editorconfig',
            'tracker/vendor/adodb/adodb-php/LICENSE.md',
            'tracker/vendor/adodb/adodb-php/README.md',
            'tracker/vendor/adodb/adodb-php/SECURITY.md',
            'tracker/vendor/adodb/adodb-php/adodb-active-record.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-active-recordx.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-csvlib.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-datadict.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-error.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-errorhandler.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-errorpear.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-exceptions.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-lib.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-loadbalancer.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-memcache.lib.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-pager.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-pear.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-perf.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-time.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-xmlschema.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb-xmlschema03.inc.php',
            'tracker/vendor/adodb/adodb-php/adodb.inc.php',
            'tracker/vendor/adodb/adodb-php/composer.json',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-access.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-db2.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-firebird.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-generic.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-ibase.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-informix.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-mssql.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-mssqlnative.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-mysql.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-oci8.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-postgres.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-sapdb.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-sqlite.inc.php',
            'tracker/vendor/adodb/adodb-php/datadict/datadict-sybase.inc.php',
            'tracker/vendor/adodb/adodb-php/docs/README.md',
            'tracker/vendor/adodb/adodb-php/docs/adodb.gif',
            'tracker/vendor/adodb/adodb-php/docs/adodb2.gif',
            'tracker/vendor/adodb/adodb-php/docs/changelog.md',
            'tracker/vendor/adodb/adodb-php/docs/changelog_v2.x.md',
            'tracker/vendor/adodb/adodb-php/docs/changelog_v3.x.md',
            'tracker/vendor/adodb/adodb-php/docs/changelog_v4+5.md',
            'tracker/vendor/adodb/adodb-php/docs/changelog_v4.x.md',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-access.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ado.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ado5.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ado_access.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ado_mssql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ads.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-borland_ibase.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-csv.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-db2.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-db2oci.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-db2ora.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-fbsql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-firebird.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ibase.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-informix.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-informix72.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-ldap.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-mssql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-mssql_n.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-mssqlnative.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-mssqlpo.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-mysqli.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-netezza.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-oci8.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-oci805.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-oci8po.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-oci8quercus.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbc.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbc_db2.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbc_mssql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbc_mssql2012.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbc_oracle.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbtp.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-odbtp_unicode.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-oracle.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_dblib.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_firebird.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_mssql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_mysql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_oci.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_pgsql.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_sqlite.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-pdo_sqlsrv.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-postgres.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-postgres64.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-postgres7.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-postgres8.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-postgres9.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-proxy.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sapdb.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sqlanywhere.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sqlite.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sqlite3.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sqlitepo.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sybase.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-sybase_ase.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-text.inc.php',
            'tracker/vendor/adodb/adodb-php/drivers/adodb-vfp.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-ar.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-bg.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-ca.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-cn.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-cz.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-da.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-de.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-en.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-eo.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-es.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-fa.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-fr.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-hu.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-id.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-it.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-nl.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-oc.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-pl.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-pt-br.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-ro.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-ru.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-sv.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-th.inc.php',
            'tracker/vendor/adodb/adodb-php/lang/adodb-uk.inc.php',
            'tracker/vendor/adodb/adodb-php/pear/Auth/Container/ADOdb.php',
            'tracker/vendor/adodb/adodb-php/pear/auth_adodb_example.php',
            'tracker/vendor/adodb/adodb-php/pear/readme.Auth.txt',
            'tracker/vendor/adodb/adodb-php/perf/perf-db2.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-informix.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-mssql.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-mssqlnative.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-mysql.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-oci8.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-postgres.inc.php',
            'tracker/vendor/adodb/adodb-php/perf/perf-sqlite3.inc.php',
            'tracker/vendor/adodb/adodb-php/phpdoc',
            'tracker/vendor/adodb/adodb-php/phpdoc.dist.xml',
            'tracker/vendor/adodb/adodb-php/pivottable.inc.php',
            'tracker/vendor/adodb/adodb-php/rsfilter.inc.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-compress-bzip2.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-compress-gzip.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-cryptsession.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-cryptsession2.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-encrypt-mcrypt.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-encrypt-md5.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-encrypt-secret.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-encrypt-sha1.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-sess.txt',
            'tracker/vendor/adodb/adodb-php/session/adodb-session-clob.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-session-clob2.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-session.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-session2.php',
            'tracker/vendor/adodb/adodb-php/session/adodb-sessions.mysql.sql',
            'tracker/vendor/adodb/adodb-php/session/adodb-sessions.oracle.clob.sql',
            'tracker/vendor/adodb/adodb-php/session/adodb-sessions.oracle.sql',
            'tracker/vendor/adodb/adodb-php/session/crypt.inc.php',
            'tracker/vendor/adodb/adodb-php/session/old/adodb-cryptsession.php',
            'tracker/vendor/adodb/adodb-php/session/old/adodb-session-clob.php',
            'tracker/vendor/adodb/adodb-php/session/old/adodb-session.php',
            'tracker/vendor/adodb/adodb-php/session/old/crypt.inc.php',
            'tracker/vendor/adodb/adodb-php/session/session_schema.xml',
            'tracker/vendor/adodb/adodb-php/session/session_schema2.xml',
            'tracker/vendor/adodb/adodb-php/toexport.inc.php',
            'tracker/vendor/adodb/adodb-php/tohtml.inc.php',
            'tracker/vendor/adodb/adodb-php/xmlschema.dtd',
            'tracker/vendor/adodb/adodb-php/xmlschema03.dtd',
            'tracker/vendor/adodb/adodb-php/xsl/convert-0.1-0.2.xsl',
            'tracker/vendor/adodb/adodb-php/xsl/convert-0.1-0.3.xsl',
            'tracker/vendor/adodb/adodb-php/xsl/convert-0.2-0.1.xsl',
            'tracker/vendor/adodb/adodb-php/xsl/convert-0.2-0.3.xsl',
            'tracker/vendor/adodb/adodb-php/xsl/remove-0.2.xsl',
            'tracker/vendor/adodb/adodb-php/xsl/remove-0.3.xsl',
            'tracker/vendor/autoload.php',
            'tracker/vendor/bin/phpunit',
            'tracker/vendor/bin/phpunit.bat',
            'tracker/vendor/composer/ClassLoader.php',
            'tracker/vendor/composer/InstalledVersions.php',
            'tracker/vendor/composer/LICENSE',
            'tracker/vendor/composer/autoload_classmap.php',
            'tracker/vendor/composer/autoload_files.php',
            'tracker/vendor/composer/autoload_namespaces.php',
            'tracker/vendor/composer/autoload_psr4.php',
            'tracker/vendor/composer/autoload_real.php',
            'tracker/vendor/composer/autoload_static.php',
            'tracker/vendor/composer/installed.json',
            'tracker/vendor/composer/installed.php',
            'tracker/vendor/composer/platform_check.php',
            'tracker/vendor/dapphp/securimage/.gitattributes',
            'tracker/vendor/dapphp/securimage/.gitignore',
            'tracker/vendor/dapphp/securimage/.htaccess',
            'tracker/vendor/dapphp/securimage/AHGBold.ttf',
            'tracker/vendor/dapphp/securimage/LICENSE.txt',
            'tracker/vendor/dapphp/securimage/README.FONT.txt',
            'tracker/vendor/dapphp/securimage/README.md',
            'tracker/vendor/dapphp/securimage/README.txt',
            'tracker/vendor/dapphp/securimage/WavFile.php',
            'tracker/vendor/dapphp/securimage/audio/.htaccess',
            'tracker/vendor/dapphp/securimage/audio/en/0.wav',
            'tracker/vendor/dapphp/securimage/audio/en/1.wav',
            'tracker/vendor/dapphp/securimage/audio/en/10.wav',
            'tracker/vendor/dapphp/securimage/audio/en/11.wav',
            'tracker/vendor/dapphp/securimage/audio/en/12.wav',
            'tracker/vendor/dapphp/securimage/audio/en/13.wav',
            'tracker/vendor/dapphp/securimage/audio/en/14.wav',
            'tracker/vendor/dapphp/securimage/audio/en/15.wav',
            'tracker/vendor/dapphp/securimage/audio/en/16.wav',
            'tracker/vendor/dapphp/securimage/audio/en/17.wav',
            'tracker/vendor/dapphp/securimage/audio/en/18.wav',
            'tracker/vendor/dapphp/securimage/audio/en/19.wav',
            'tracker/vendor/dapphp/securimage/audio/en/2.wav',
            'tracker/vendor/dapphp/securimage/audio/en/20.wav',
            'tracker/vendor/dapphp/securimage/audio/en/3.wav',
            'tracker/vendor/dapphp/securimage/audio/en/4.wav',
            'tracker/vendor/dapphp/securimage/audio/en/5.wav',
            'tracker/vendor/dapphp/securimage/audio/en/6.wav',
            'tracker/vendor/dapphp/securimage/audio/en/7.wav',
            'tracker/vendor/dapphp/securimage/audio/en/8.wav',
            'tracker/vendor/dapphp/securimage/audio/en/9.wav',
            'tracker/vendor/dapphp/securimage/audio/en/A.wav',
            'tracker/vendor/dapphp/securimage/audio/en/B.wav',
            'tracker/vendor/dapphp/securimage/audio/en/C.wav',
            'tracker/vendor/dapphp/securimage/audio/en/D.wav',
            'tracker/vendor/dapphp/securimage/audio/en/E.wav',
            'tracker/vendor/dapphp/securimage/audio/en/F.wav',
            'tracker/vendor/dapphp/securimage/audio/en/G.wav',
            'tracker/vendor/dapphp/securimage/audio/en/H.wav',
            'tracker/vendor/dapphp/securimage/audio/en/I.wav',
            'tracker/vendor/dapphp/securimage/audio/en/J.wav',
            'tracker/vendor/dapphp/securimage/audio/en/K.wav',
            'tracker/vendor/dapphp/securimage/audio/en/L.wav',
            'tracker/vendor/dapphp/securimage/audio/en/M.wav',
            'tracker/vendor/dapphp/securimage/audio/en/MINUS.wav',
            'tracker/vendor/dapphp/securimage/audio/en/N.wav',
            'tracker/vendor/dapphp/securimage/audio/en/O.wav',
            'tracker/vendor/dapphp/securimage/audio/en/P.wav',
            'tracker/vendor/dapphp/securimage/audio/en/PLUS.wav',
            'tracker/vendor/dapphp/securimage/audio/en/Q.wav',
            'tracker/vendor/dapphp/securimage/audio/en/R.wav',
            'tracker/vendor/dapphp/securimage/audio/en/S.wav',
            'tracker/vendor/dapphp/securimage/audio/en/T.wav',
            'tracker/vendor/dapphp/securimage/audio/en/TIMES.wav',
            'tracker/vendor/dapphp/securimage/audio/en/U.wav',
            'tracker/vendor/dapphp/securimage/audio/en/V.wav',
            'tracker/vendor/dapphp/securimage/audio/en/W.wav',
            'tracker/vendor/dapphp/securimage/audio/en/X.wav',
            'tracker/vendor/dapphp/securimage/audio/en/Y.wav',
            'tracker/vendor/dapphp/securimage/audio/en/Z.wav',
            'tracker/vendor/dapphp/securimage/audio/en/error.wav',
            'tracker/vendor/dapphp/securimage/audio/noise/check-point-1.wav',
            'tracker/vendor/dapphp/securimage/audio/noise/crowd-talking-1.wav',
            'tracker/vendor/dapphp/securimage/audio/noise/crowd-talking-6.wav',
            'tracker/vendor/dapphp/securimage/audio/noise/crowd-talking-7.wav',
            'tracker/vendor/dapphp/securimage/audio/noise/kids-playing-1.wav',
            'tracker/vendor/dapphp/securimage/backgrounds/bg3.jpg',
            'tracker/vendor/dapphp/securimage/backgrounds/bg4.jpg',
            'tracker/vendor/dapphp/securimage/backgrounds/bg5.jpg',
            'tracker/vendor/dapphp/securimage/backgrounds/bg6.png',
            'tracker/vendor/dapphp/securimage/composer.json',
            'tracker/vendor/dapphp/securimage/database/.htaccess',
            'tracker/vendor/dapphp/securimage/database/index.html',
            'tracker/vendor/dapphp/securimage/database/securimage.sq3',
            'tracker/vendor/dapphp/securimage/images/audio_icon.png',
            'tracker/vendor/dapphp/securimage/images/loading.png',
            'tracker/vendor/dapphp/securimage/images/refresh.png',
            'tracker/vendor/dapphp/securimage/securimage.css',
            'tracker/vendor/dapphp/securimage/securimage.js',
            'tracker/vendor/dapphp/securimage/securimage.php',
            'tracker/vendor/dapphp/securimage/securimage_play.php',
            'tracker/vendor/dapphp/securimage/securimage_play.swf',
            'tracker/vendor/dapphp/securimage/securimage_show.php',
            'tracker/vendor/dapphp/securimage/web.config',
            'tracker/vendor/dapphp/securimage/words/words.txt',
            'tracker/vendor/doctrine/instantiator/.doctrine-project.json',
            'tracker/vendor/doctrine/instantiator/CONTRIBUTING.md',
            'tracker/vendor/doctrine/instantiator/LICENSE',
            'tracker/vendor/doctrine/instantiator/README.md',
            'tracker/vendor/doctrine/instantiator/composer.json',
            'tracker/vendor/doctrine/instantiator/docs/en/index.rst',
            'tracker/vendor/doctrine/instantiator/docs/en/sidebar.rst',
            'tracker/vendor/doctrine/instantiator/psalm.xml',
            'tracker/vendor/doctrine/instantiator/src/Doctrine/Instantiator/Exception/ExceptionInterface.php',
            'tracker/vendor/doctrine/instantiator/src/Doctrine/Instantiator/Exception/InvalidArgumentException.php',
            'tracker/vendor/doctrine/instantiator/src/Doctrine/Instantiator/Exception/UnexpectedValueException.php',
            'tracker/vendor/doctrine/instantiator/src/Doctrine/Instantiator/Instantiator.php',
            'tracker/vendor/doctrine/instantiator/src/Doctrine/Instantiator/InstantiatorInterface.php',
            'tracker/vendor/erusev/parsedown/LICENSE.txt',
            'tracker/vendor/erusev/parsedown/Parsedown.php',
            'tracker/vendor/erusev/parsedown/README.md',
            'tracker/vendor/erusev/parsedown/composer.json',
            'tracker/vendor/ezyang/htmlpurifier/CHANGELOG.md',
            'tracker/vendor/ezyang/htmlpurifier/CREDITS',
            'tracker/vendor/ezyang/htmlpurifier/LICENSE',
            'tracker/vendor/ezyang/htmlpurifier/README.md',
            'tracker/vendor/ezyang/htmlpurifier/VERSION',
            'tracker/vendor/ezyang/htmlpurifier/composer.json',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.auto.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.autoload-legacy.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.autoload.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.composer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.func.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.includes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.kses.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.path.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier.safe-includes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Arborize.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrCollections.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/AlphaValue.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Background.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/BackgroundPosition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Border.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Color.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Composite.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/DenyElementDecorator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Filter.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Font.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/FontFamily.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Ident.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/ImportantDecorator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Length.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/ListStyle.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Multiple.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Number.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Percentage.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/TextDecoration.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/URI.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Clone.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Enum.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Bool.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Class.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Color.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/ContentEditable.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/FrameTarget.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/ID.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Length.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/LinkTypes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/MultiLength.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Nmtokens.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Pixels.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Integer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Lang.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Switch.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Text.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/Email.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/Email/SimpleCheck.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/Host.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/IPv4.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/IPv6.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Background.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/BdoDir.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/BgColor.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/BoolToCSS.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Border.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/EnumToCSS.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/ImgRequired.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/ImgSpace.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Input.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Lang.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Length.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Name.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/NameSync.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Nofollow.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/SafeEmbed.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/SafeObject.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/SafeParam.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/ScriptRequired.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/TargetBlank.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/TargetNoopener.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/TargetNoreferrer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Textarea.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrTypes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/AttrValidator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Bootstrap.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/CSSDefinition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Chameleon.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Custom.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Empty.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/List.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Optional.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Required.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/StrictBlockquote.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Table.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Config.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Builder/ConfigSchema.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Builder/Xml.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Exception.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Interchange.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Interchange/Directive.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Interchange/Id.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/InterchangeBuilder.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Validator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/ValidatorAtom.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema.ser',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.AllowedClasses.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.AllowedFrameTargets.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.AllowedRel.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.AllowedRev.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.ClassUseCDATA.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.DefaultImageAlt.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.DefaultInvalidImage.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.DefaultInvalidImageAlt.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.DefaultTextDir.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.EnableID.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.ForbiddenClasses.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.ID.HTML5.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.IDBlacklist.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.IDBlacklistRegexp.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.IDPrefix.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Attr.IDPrefixLocal.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.AutoParagraph.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.Custom.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.DisplayLinkURI.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.Linkify.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.PurifierLinkify.DocURL.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.PurifierLinkify.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.RemoveEmpty.Predicate.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.RemoveEmpty.RemoveNbsp.Exceptions.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.RemoveEmpty.RemoveNbsp.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.RemoveEmpty.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/AutoFormat.RemoveSpansWithoutAttributes.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.AllowDuplicates.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.AllowImportant.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.AllowTricky.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.AllowedFonts.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.AllowedProperties.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.DefinitionRev.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.ForbiddenProperties.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.MaxImgLength.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.Proprietary.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/CSS.Trusted.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Cache.DefinitionImpl.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Cache.SerializerPath.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Cache.SerializerPermissions.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.AggressivelyFixLt.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.AggressivelyRemoveScript.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.AllowHostnameUnderscore.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.AllowParseManyTags.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.CollectErrors.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.ColorKeywords.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.ConvertDocumentToFragment.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.DirectLexLineNumberSyncInterval.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.DisableExcludes.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.EnableIDNA.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.Encoding.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.EscapeInvalidChildren.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.EscapeInvalidTags.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.EscapeNonASCIICharacters.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.HiddenElements.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.Language.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.LegacyEntityDecoder.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.LexerImpl.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.MaintainLineNumbers.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.NormalizeNewlines.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.RemoveInvalidImg.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.RemoveProcessingInstructions.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Core.RemoveScriptContents.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Filter.Custom.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Filter.ExtractStyleBlocks.Escaping.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Filter.ExtractStyleBlocks.Scope.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Filter.ExtractStyleBlocks.TidyImpl.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Filter.ExtractStyleBlocks.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Filter.YouTube.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Allowed.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.AllowedAttributes.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.AllowedComments.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.AllowedCommentsRegexp.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.AllowedElements.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.AllowedModules.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Attr.Name.UseCDATA.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.BlockWrapper.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.CoreModules.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.CustomDoctype.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.DefinitionID.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.DefinitionRev.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Doctype.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.FlashAllowFullScreen.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.ForbiddenAttributes.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.ForbiddenElements.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Forms.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.MaxImgLength.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Nofollow.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Parent.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Proprietary.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.SafeEmbed.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.SafeIframe.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.SafeObject.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.SafeScripting.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Strict.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.TargetBlank.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.TargetNoopener.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.TargetNoreferrer.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.TidyAdd.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.TidyLevel.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.TidyRemove.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.Trusted.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/HTML.XHTML.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Output.CommentScriptContents.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Output.FixInnerHTML.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Output.FlashCompat.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Output.Newline.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Output.SortAttr.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Output.TidyFormat.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/Test.ForceNoIconv.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.AllowedSchemes.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.Base.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.DefaultScheme.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.DefinitionID.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.DefinitionRev.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.Disable.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.DisableExternal.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.DisableExternalResources.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.DisableResources.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.Host.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.HostBlacklist.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.MakeAbsolute.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.Munge.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.MungeResources.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.MungeSecretKey.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.OverrideAllowedSchemes.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/URI.SafeIframeRegexp.txt',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/schema/info.ini',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ContentSets.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Context.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Definition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator/Cleanup.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator/Memory.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator/Template.php.in',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Null.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Serializer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Serializer/README',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCacheFactory.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Doctype.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/DoctypeRegistry.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ElementDef.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Encoder.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/EntityLookup.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/EntityLookup/entities.ser',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/EntityParser.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ErrorCollector.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/ErrorStruct.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Exception.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Filter.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Filter/ExtractStyleBlocks.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Filter/YouTube.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Generator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLDefinition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Bdo.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/CommonAttributes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Edit.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Forms.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Hypertext.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Iframe.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Image.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Legacy.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/List.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Name.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Nofollow.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/NonXMLCommonAttributes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Object.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Presentation.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Proprietary.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Ruby.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/SafeEmbed.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/SafeObject.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/SafeScripting.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Scripting.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/StyleAttribute.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tables.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Target.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/TargetBlank.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/TargetNoopener.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/TargetNoreferrer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Text.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Name.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Proprietary.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Strict.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Transitional.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/XHTML.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/XHTMLAndHTML4.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/XMLCommonAttributes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModuleManager.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/IDAccumulator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/AutoParagraph.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/DisplayLinkURI.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/Linkify.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/PurifierLinkify.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/RemoveEmpty.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/RemoveSpansWithoutAttributes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Injector/SafeObject.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Language.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Language/messages/en.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/LanguageFactory.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Length.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Lexer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Lexer/DOMLex.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Lexer/DirectLex.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Lexer/PH5P.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Node.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Node/Comment.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Node/Element.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Node/Text.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/PercentEncoder.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Printer.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Printer/CSSDefinition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Printer/ConfigForm.css',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Printer/ConfigForm.js',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Printer/ConfigForm.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Printer/HTMLDefinition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/PropertyList.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/PropertyListIterator.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Queue.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/Composite.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/Core.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/FixNesting.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/MakeWellFormed.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/RemoveForeignElements.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/ValidateAttributes.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/StringHash.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/StringHashParser.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/TagTransform.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/TagTransform/Font.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/TagTransform/Simple.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token/Comment.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token/Empty.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token/End.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token/Start.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token/Tag.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Token/Text.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/TokenFactory.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URI.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIDefinition.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/DisableExternal.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/DisableExternalResources.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/DisableResources.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/HostBlacklist.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/MakeAbsolute.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/Munge.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/SafeIframe.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIParser.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/data.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/file.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/ftp.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/http.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/https.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/mailto.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/news.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/nntp.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/tel.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/URISchemeRegistry.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/UnitConverter.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/VarParser.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/VarParser/Flexible.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/VarParser/Native.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/VarParserException.php',
            'tracker/vendor/ezyang/htmlpurifier/library/HTMLPurifier/Zipper.php',
            'tracker/vendor/guzzlehttp/guzzle/CHANGELOG.md',
            'tracker/vendor/guzzlehttp/guzzle/LICENSE',
            'tracker/vendor/guzzlehttp/guzzle/README.md',
            'tracker/vendor/guzzlehttp/guzzle/UPGRADING.md',
            'tracker/vendor/guzzlehttp/guzzle/composer.json',
            'tracker/vendor/guzzlehttp/guzzle/src/BodySummarizer.php',
            'tracker/vendor/guzzlehttp/guzzle/src/BodySummarizerInterface.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Client.php',
            'tracker/vendor/guzzlehttp/guzzle/src/ClientInterface.php',
            'tracker/vendor/guzzlehttp/guzzle/src/ClientTrait.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Cookie/CookieJar.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Cookie/CookieJarInterface.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Cookie/FileCookieJar.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Cookie/SessionCookieJar.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Cookie/SetCookie.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/BadResponseException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/ClientException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/ConnectException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/GuzzleException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/InvalidArgumentException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/RequestException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/ServerException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/TooManyRedirectsException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Exception/TransferException.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/CurlFactory.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/CurlFactoryInterface.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/CurlHandler.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/CurlMultiHandler.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/EasyHandle.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/HeaderProcessor.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/MockHandler.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/Proxy.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Handler/StreamHandler.php',
            'tracker/vendor/guzzlehttp/guzzle/src/HandlerStack.php',
            'tracker/vendor/guzzlehttp/guzzle/src/MessageFormatter.php',
            'tracker/vendor/guzzlehttp/guzzle/src/MessageFormatterInterface.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Middleware.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Pool.php',
            'tracker/vendor/guzzlehttp/guzzle/src/PrepareBodyMiddleware.php',
            'tracker/vendor/guzzlehttp/guzzle/src/RedirectMiddleware.php',
            'tracker/vendor/guzzlehttp/guzzle/src/RequestOptions.php',
            'tracker/vendor/guzzlehttp/guzzle/src/RetryMiddleware.php',
            'tracker/vendor/guzzlehttp/guzzle/src/TransferStats.php',
            'tracker/vendor/guzzlehttp/guzzle/src/Utils.php',
            'tracker/vendor/guzzlehttp/guzzle/src/functions.php',
            'tracker/vendor/guzzlehttp/guzzle/src/functions_include.php',
            'tracker/vendor/guzzlehttp/promises/CHANGELOG.md',
            'tracker/vendor/guzzlehttp/promises/LICENSE',
            'tracker/vendor/guzzlehttp/promises/README.md',
            'tracker/vendor/guzzlehttp/promises/composer.json',
            'tracker/vendor/guzzlehttp/promises/src/AggregateException.php',
            'tracker/vendor/guzzlehttp/promises/src/CancellationException.php',
            'tracker/vendor/guzzlehttp/promises/src/Coroutine.php',
            'tracker/vendor/guzzlehttp/promises/src/Create.php',
            'tracker/vendor/guzzlehttp/promises/src/Each.php',
            'tracker/vendor/guzzlehttp/promises/src/EachPromise.php',
            'tracker/vendor/guzzlehttp/promises/src/FulfilledPromise.php',
            'tracker/vendor/guzzlehttp/promises/src/Is.php',
            'tracker/vendor/guzzlehttp/promises/src/Promise.php',
            'tracker/vendor/guzzlehttp/promises/src/PromiseInterface.php',
            'tracker/vendor/guzzlehttp/promises/src/PromisorInterface.php',
            'tracker/vendor/guzzlehttp/promises/src/RejectedPromise.php',
            'tracker/vendor/guzzlehttp/promises/src/RejectionException.php',
            'tracker/vendor/guzzlehttp/promises/src/TaskQueue.php',
            'tracker/vendor/guzzlehttp/promises/src/TaskQueueInterface.php',
            'tracker/vendor/guzzlehttp/promises/src/Utils.php',
            'tracker/vendor/guzzlehttp/psr7/CHANGELOG.md',
            'tracker/vendor/guzzlehttp/psr7/LICENSE',
            'tracker/vendor/guzzlehttp/psr7/README.md',
            'tracker/vendor/guzzlehttp/psr7/composer.json',
            'tracker/vendor/guzzlehttp/psr7/src/AppendStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/BufferStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/CachingStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/DroppingStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/Exception/MalformedUriException.php',
            'tracker/vendor/guzzlehttp/psr7/src/FnStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/Header.php',
            'tracker/vendor/guzzlehttp/psr7/src/HttpFactory.php',
            'tracker/vendor/guzzlehttp/psr7/src/InflateStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/LazyOpenStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/LimitStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/Message.php',
            'tracker/vendor/guzzlehttp/psr7/src/MessageTrait.php',
            'tracker/vendor/guzzlehttp/psr7/src/MimeType.php',
            'tracker/vendor/guzzlehttp/psr7/src/MultipartStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/NoSeekStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/PumpStream.php',
            'tracker/vendor/guzzlehttp/psr7/src/Query.php',
            'tracker/vendor/guzzlehttp/psr7/src/Request.php',
            'tracker/vendor/guzzlehttp/psr7/src/Response.php',
            'tracker/vendor/guzzlehttp/psr7/src/Rfc7230.php',
            'tracker/vendor/guzzlehttp/psr7/src/ServerRequest.php',
            'tracker/vendor/guzzlehttp/psr7/src/Stream.php',
            'tracker/vendor/guzzlehttp/psr7/src/StreamDecoratorTrait.php',
            'tracker/vendor/guzzlehttp/psr7/src/StreamWrapper.php',
            'tracker/vendor/guzzlehttp/psr7/src/UploadedFile.php',
            'tracker/vendor/guzzlehttp/psr7/src/Uri.php',
            'tracker/vendor/guzzlehttp/psr7/src/UriComparator.php',
            'tracker/vendor/guzzlehttp/psr7/src/UriNormalizer.php',
            'tracker/vendor/guzzlehttp/psr7/src/UriResolver.php',
            'tracker/vendor/guzzlehttp/psr7/src/Utils.php',
            'tracker/vendor/myclabs/deep-copy/.github/FUNDING.yml',
            'tracker/vendor/myclabs/deep-copy/.github/workflows/ci.yaml',
            'tracker/vendor/myclabs/deep-copy/LICENSE',
            'tracker/vendor/myclabs/deep-copy/README.md',
            'tracker/vendor/myclabs/deep-copy/composer.json',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/DeepCopy.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Exception/CloneException.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Exception/PropertyException.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/ChainableFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/Doctrine/DoctrineCollectionFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/Doctrine/DoctrineEmptyCollectionFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/Doctrine/DoctrineProxyFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/Filter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/KeepFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/ReplaceFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Filter/SetNullFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Matcher/Doctrine/DoctrineProxyMatcher.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Matcher/Matcher.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Matcher/PropertyMatcher.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Matcher/PropertyNameMatcher.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Matcher/PropertyTypeMatcher.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/Reflection/ReflectionHelper.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/Date/DateIntervalFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/ReplaceFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/ShallowCopyFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/Spl/ArrayObjectFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/Spl/SplDoublyLinkedList.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/Spl/SplDoublyLinkedListFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeFilter/TypeFilter.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/TypeMatcher/TypeMatcher.php',
            'tracker/vendor/myclabs/deep-copy/src/DeepCopy/deep_copy.php',
            'tracker/vendor/nikic/fast-route/.gitignore',
            'tracker/vendor/nikic/fast-route/.hhconfig',
            'tracker/vendor/nikic/fast-route/.travis.yml',
            'tracker/vendor/nikic/fast-route/FastRoute.hhi',
            'tracker/vendor/nikic/fast-route/LICENSE',
            'tracker/vendor/nikic/fast-route/README.md',
            'tracker/vendor/nikic/fast-route/composer.json',
            'tracker/vendor/nikic/fast-route/phpunit.xml',
            'tracker/vendor/nikic/fast-route/psalm.xml',
            'tracker/vendor/nikic/fast-route/src/BadRouteException.php',
            'tracker/vendor/nikic/fast-route/src/DataGenerator.php',
            'tracker/vendor/nikic/fast-route/src/DataGenerator/CharCountBased.php',
            'tracker/vendor/nikic/fast-route/src/DataGenerator/GroupCountBased.php',
            'tracker/vendor/nikic/fast-route/src/DataGenerator/GroupPosBased.php',
            'tracker/vendor/nikic/fast-route/src/DataGenerator/MarkBased.php',
            'tracker/vendor/nikic/fast-route/src/DataGenerator/RegexBasedAbstract.php',
            'tracker/vendor/nikic/fast-route/src/Dispatcher.php',
            'tracker/vendor/nikic/fast-route/src/Dispatcher/CharCountBased.php',
            'tracker/vendor/nikic/fast-route/src/Dispatcher/GroupCountBased.php',
            'tracker/vendor/nikic/fast-route/src/Dispatcher/GroupPosBased.php',
            'tracker/vendor/nikic/fast-route/src/Dispatcher/MarkBased.php',
            'tracker/vendor/nikic/fast-route/src/Dispatcher/RegexBasedAbstract.php',
            'tracker/vendor/nikic/fast-route/src/Route.php',
            'tracker/vendor/nikic/fast-route/src/RouteCollector.php',
            'tracker/vendor/nikic/fast-route/src/RouteParser.php',
            'tracker/vendor/nikic/fast-route/src/RouteParser/Std.php',
            'tracker/vendor/nikic/fast-route/src/bootstrap.php',
            'tracker/vendor/nikic/fast-route/src/functions.php',
            'tracker/vendor/nikic/fast-route/test/Dispatcher/CharCountBasedTest.php',
            'tracker/vendor/nikic/fast-route/test/Dispatcher/DispatcherTest.php',
            'tracker/vendor/nikic/fast-route/test/Dispatcher/GroupCountBasedTest.php',
            'tracker/vendor/nikic/fast-route/test/Dispatcher/GroupPosBasedTest.php',
            'tracker/vendor/nikic/fast-route/test/Dispatcher/MarkBasedTest.php',
            'tracker/vendor/nikic/fast-route/test/HackTypechecker/HackTypecheckerTest.php',
            'tracker/vendor/nikic/fast-route/test/HackTypechecker/fixtures/all_options.php',
            'tracker/vendor/nikic/fast-route/test/HackTypechecker/fixtures/empty_options.php',
            'tracker/vendor/nikic/fast-route/test/HackTypechecker/fixtures/no_options.php',
            'tracker/vendor/nikic/fast-route/test/RouteCollectorTest.php',
            'tracker/vendor/nikic/fast-route/test/RouteParser/StdTest.php',
            'tracker/vendor/nikic/fast-route/test/bootstrap.php',
            'tracker/vendor/phar-io/manifest/CHANGELOG.md',
            'tracker/vendor/phar-io/manifest/LICENSE',
            'tracker/vendor/phar-io/manifest/README.md',
            'tracker/vendor/phar-io/manifest/composer.json',
            'tracker/vendor/phar-io/manifest/composer.lock',
            'tracker/vendor/phar-io/manifest/src/ManifestDocumentMapper.php',
            'tracker/vendor/phar-io/manifest/src/ManifestLoader.php',
            'tracker/vendor/phar-io/manifest/src/ManifestSerializer.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/ElementCollectionException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/Exception.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/InvalidApplicationNameException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/InvalidEmailException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/InvalidUrlException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/ManifestDocumentException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/ManifestDocumentLoadingException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/ManifestDocumentMapperException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/ManifestElementException.php',
            'tracker/vendor/phar-io/manifest/src/exceptions/ManifestLoaderException.php',
            'tracker/vendor/phar-io/manifest/src/values/Application.php',
            'tracker/vendor/phar-io/manifest/src/values/ApplicationName.php',
            'tracker/vendor/phar-io/manifest/src/values/Author.php',
            'tracker/vendor/phar-io/manifest/src/values/AuthorCollection.php',
            'tracker/vendor/phar-io/manifest/src/values/AuthorCollectionIterator.php',
            'tracker/vendor/phar-io/manifest/src/values/BundledComponent.php',
            'tracker/vendor/phar-io/manifest/src/values/BundledComponentCollection.php',
            'tracker/vendor/phar-io/manifest/src/values/BundledComponentCollectionIterator.php',
            'tracker/vendor/phar-io/manifest/src/values/CopyrightInformation.php',
            'tracker/vendor/phar-io/manifest/src/values/Email.php',
            'tracker/vendor/phar-io/manifest/src/values/Extension.php',
            'tracker/vendor/phar-io/manifest/src/values/Library.php',
            'tracker/vendor/phar-io/manifest/src/values/License.php',
            'tracker/vendor/phar-io/manifest/src/values/Manifest.php',
            'tracker/vendor/phar-io/manifest/src/values/PhpExtensionRequirement.php',
            'tracker/vendor/phar-io/manifest/src/values/PhpVersionRequirement.php',
            'tracker/vendor/phar-io/manifest/src/values/Requirement.php',
            'tracker/vendor/phar-io/manifest/src/values/RequirementCollection.php',
            'tracker/vendor/phar-io/manifest/src/values/RequirementCollectionIterator.php',
            'tracker/vendor/phar-io/manifest/src/values/Type.php',
            'tracker/vendor/phar-io/manifest/src/values/Url.php',
            'tracker/vendor/phar-io/manifest/src/xml/AuthorElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/AuthorElementCollection.php',
            'tracker/vendor/phar-io/manifest/src/xml/BundlesElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/ComponentElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/ComponentElementCollection.php',
            'tracker/vendor/phar-io/manifest/src/xml/ContainsElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/CopyrightElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/ElementCollection.php',
            'tracker/vendor/phar-io/manifest/src/xml/ExtElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/ExtElementCollection.php',
            'tracker/vendor/phar-io/manifest/src/xml/ExtensionElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/LicenseElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/ManifestDocument.php',
            'tracker/vendor/phar-io/manifest/src/xml/ManifestElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/PhpElement.php',
            'tracker/vendor/phar-io/manifest/src/xml/RequiresElement.php',
            'tracker/vendor/phar-io/version/CHANGELOG.md',
            'tracker/vendor/phar-io/version/LICENSE',
            'tracker/vendor/phar-io/version/README.md',
            'tracker/vendor/phar-io/version/composer.json',
            'tracker/vendor/phar-io/version/src/BuildMetaData.php',
            'tracker/vendor/phar-io/version/src/PreReleaseSuffix.php',
            'tracker/vendor/phar-io/version/src/Version.php',
            'tracker/vendor/phar-io/version/src/VersionConstraintParser.php',
            'tracker/vendor/phar-io/version/src/VersionConstraintValue.php',
            'tracker/vendor/phar-io/version/src/VersionNumber.php',
            'tracker/vendor/phar-io/version/src/constraints/AbstractVersionConstraint.php',
            'tracker/vendor/phar-io/version/src/constraints/AndVersionConstraintGroup.php',
            'tracker/vendor/phar-io/version/src/constraints/AnyVersionConstraint.php',
            'tracker/vendor/phar-io/version/src/constraints/ExactVersionConstraint.php',
            'tracker/vendor/phar-io/version/src/constraints/GreaterThanOrEqualToVersionConstraint.php',
            'tracker/vendor/phar-io/version/src/constraints/OrVersionConstraintGroup.php',
            'tracker/vendor/phar-io/version/src/constraints/SpecificMajorAndMinorVersionConstraint.php',
            'tracker/vendor/phar-io/version/src/constraints/SpecificMajorVersionConstraint.php',
            'tracker/vendor/phar-io/version/src/constraints/VersionConstraint.php',
            'tracker/vendor/phar-io/version/src/exceptions/Exception.php',
            'tracker/vendor/phar-io/version/src/exceptions/InvalidPreReleaseSuffixException.php',
            'tracker/vendor/phar-io/version/src/exceptions/InvalidVersionException.php',
            'tracker/vendor/phar-io/version/src/exceptions/NoBuildMetaDataException.php',
            'tracker/vendor/phar-io/version/src/exceptions/NoPreReleaseSuffixException.php',
            'tracker/vendor/phar-io/version/src/exceptions/UnsupportedVersionConstraintException.php',
            'tracker/vendor/phpmailer/phpmailer/COMMITMENT',
            'tracker/vendor/phpmailer/phpmailer/LICENSE',
            'tracker/vendor/phpmailer/phpmailer/README.md',
            'tracker/vendor/phpmailer/phpmailer/SECURITY.md',
            'tracker/vendor/phpmailer/phpmailer/VERSION',
            'tracker/vendor/phpmailer/phpmailer/composer.json',
            'tracker/vendor/phpmailer/phpmailer/get_oauth_token.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-af.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ar.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-az.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ba.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-be.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-bg.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ca.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-cs.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-da.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-de.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-el.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-eo.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-es.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-et.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-fa.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-fi.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-fo.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-fr.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-gl.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-he.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-hi.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-hr.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-hu.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-hy.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-id.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-it.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ja.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ka.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ko.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-lt.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-lv.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-mg.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-mn.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ms.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-nb.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-nl.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-pl.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-pt.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-pt_br.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ro.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-ru.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-si.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-sk.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-sl.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-sr.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-sr_latn.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-sv.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-tl.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-tr.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-uk.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-vi.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-zh.php',
            'tracker/vendor/phpmailer/phpmailer/language/phpmailer.lang-zh_cn.php',
            'tracker/vendor/phpmailer/phpmailer/src/DSNConfigurator.php',
            'tracker/vendor/phpmailer/phpmailer/src/Exception.php',
            'tracker/vendor/phpmailer/phpmailer/src/OAuth.php',
            'tracker/vendor/phpmailer/phpmailer/src/OAuthTokenProvider.php',
            'tracker/vendor/phpmailer/phpmailer/src/PHPMailer.php',
            'tracker/vendor/phpmailer/phpmailer/src/POP3.php',
            'tracker/vendor/phpmailer/phpmailer/src/SMTP.php',
            'tracker/vendor/phpunit/php-code-coverage/.gitattributes',
            'tracker/vendor/phpunit/php-code-coverage/.github/CONTRIBUTING.md',
            'tracker/vendor/phpunit/php-code-coverage/.github/FUNDING.yml',
            'tracker/vendor/phpunit/php-code-coverage/.github/ISSUE_TEMPLATE.md',
            'tracker/vendor/phpunit/php-code-coverage/.gitignore',
            'tracker/vendor/phpunit/php-code-coverage/.php_cs.dist',
            'tracker/vendor/phpunit/php-code-coverage/.travis.yml',
            'tracker/vendor/phpunit/php-code-coverage/ChangeLog.md',
            'tracker/vendor/phpunit/php-code-coverage/LICENSE',
            'tracker/vendor/phpunit/php-code-coverage/README.md',
            'tracker/vendor/phpunit/php-code-coverage/build.xml',
            'tracker/vendor/phpunit/php-code-coverage/composer.json',
            'tracker/vendor/phpunit/php-code-coverage/phive.xml',
            'tracker/vendor/phpunit/php-code-coverage/phpunit.xml',
            'tracker/vendor/phpunit/php-code-coverage/src/CodeCoverage.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Driver/Driver.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Driver/PCOV.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Driver/PHPDBG.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Driver/Xdebug.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Exception/CoveredCodeNotExecutedException.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Exception/Exception.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Exception/InvalidArgumentException.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Exception/MissingCoversAnnotationException.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Exception/RuntimeException.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Exception/UnintentionallyCoveredCodeException.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Filter.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Node/AbstractNode.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Node/Builder.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Node/Directory.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Node/File.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Node/Iterator.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Clover.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Crap4j.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Facade.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Dashboard.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Directory.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/File.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/coverage_bar.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/css/bootstrap.min.css',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/css/custom.css',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/css/nv.d3.min.css',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/css/octicons.css',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/css/style.css',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/dashboard.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/directory.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/directory_item.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/file.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/file_item.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/icons/file-code.svg',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/icons/file-directory.svg',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/js/bootstrap.min.js',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/js/d3.min.js',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/js/file.js',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/js/jquery.min.js',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/js/nv.d3.min.js',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/js/popper.min.js',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Html/Renderer/Template/method_item.html.dist',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/PHP.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Text.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/BuildInformation.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Coverage.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Directory.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Facade.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/File.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Method.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Node.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Project.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Report.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Source.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Tests.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Totals.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Report/Xml/Unit.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Util.php',
            'tracker/vendor/phpunit/php-code-coverage/src/Version.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/TestCase.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/BankAccount-clover.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/BankAccount-crap4j.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/BankAccount-text.txt',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/BankAccount.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/BankAccountTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageClassExtendedTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageClassTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageFunctionParenthesesTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageFunctionParenthesesWhitespaceTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageFunctionTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageMethodOneLineAnnotationTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageMethodParenthesesTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageMethodParenthesesWhitespaceTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageMethodTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageNoneTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageNotPrivateTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageNotProtectedTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageNotPublicTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageNothingTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoveragePrivateTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageProtectedTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoveragePublicTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoverageTwoDefaultClassAnnotations.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoveredClass.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/CoveredFunction.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Crash.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageClassExtendedTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageClassTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageCoversClassPublicTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageCoversClassTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageMethodTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageNotPrivateTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageNotProtectedTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageNotPublicTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoveragePrivateTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoverageProtectedTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoveragePublicTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NamespaceCoveredClass.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/NotExistingCoveredElementTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForBankAccount/BankAccount.php.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForBankAccount/dashboard.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForBankAccount/index.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForClassWithAnonymousFunction/dashboard.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForClassWithAnonymousFunction/index.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForClassWithAnonymousFunction/source_with_class_and_anonymous_function.php.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForFileWithIgnoredLines/dashboard.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForFileWithIgnoredLines/index.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/HTML/CoverageForFileWithIgnoredLines/source_with_ignore.php.html',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/XML/CoverageForBankAccount/BankAccount.php.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/XML/CoverageForBankAccount/index.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/XML/CoverageForClassWithAnonymousFunction/index.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/XML/CoverageForClassWithAnonymousFunction/source_with_class_and_anonymous_function.php.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/XML/CoverageForFileWithIgnoredLines/index.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/Report/XML/CoverageForFileWithIgnoredLines/source_with_ignore.php.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/class-with-anonymous-function-clover.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/class-with-anonymous-function-crap4j.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/class-with-anonymous-function-text.txt',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/ignored-lines-clover.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/ignored-lines-crap4j.xml',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/ignored-lines-text.txt',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_with_class_and_anonymous_function.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_with_ignore.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_with_namespace.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_with_oneline_annotations.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_with_use_statements.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_without_ignore.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/_files/source_without_namespace.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/bootstrap.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/BuilderTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/CloverTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/CodeCoverageTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/Crap4jTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/Exception/UnintentionallyCoveredCodeExceptionTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/FilterTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/HTMLTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/TextTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/UtilTest.php',
            'tracker/vendor/phpunit/php-code-coverage/tests/tests/XmlTest.php',
            'tracker/vendor/phpunit/php-file-iterator/.gitattributes',
            'tracker/vendor/phpunit/php-file-iterator/.github/stale.yml',
            'tracker/vendor/phpunit/php-file-iterator/.gitignore',
            'tracker/vendor/phpunit/php-file-iterator/.php_cs.dist',
            'tracker/vendor/phpunit/php-file-iterator/.travis.yml',
            'tracker/vendor/phpunit/php-file-iterator/ChangeLog.md',
            'tracker/vendor/phpunit/php-file-iterator/LICENSE',
            'tracker/vendor/phpunit/php-file-iterator/README.md',
            'tracker/vendor/phpunit/php-file-iterator/composer.json',
            'tracker/vendor/phpunit/php-file-iterator/phpunit.xml',
            'tracker/vendor/phpunit/php-file-iterator/src/Facade.php',
            'tracker/vendor/phpunit/php-file-iterator/src/Factory.php',
            'tracker/vendor/phpunit/php-file-iterator/src/Iterator.php',
            'tracker/vendor/phpunit/php-file-iterator/tests/FactoryTest.php',
            'tracker/vendor/phpunit/php-text-template/.gitattributes',
            'tracker/vendor/phpunit/php-text-template/.gitignore',
            'tracker/vendor/phpunit/php-text-template/LICENSE',
            'tracker/vendor/phpunit/php-text-template/README.md',
            'tracker/vendor/phpunit/php-text-template/composer.json',
            'tracker/vendor/phpunit/php-text-template/src/Template.php',
            'tracker/vendor/phpunit/php-timer/.gitattributes',
            'tracker/vendor/phpunit/php-timer/.github/FUNDING.yml',
            'tracker/vendor/phpunit/php-timer/.github/stale.yml',
            'tracker/vendor/phpunit/php-timer/.gitignore',
            'tracker/vendor/phpunit/php-timer/.php_cs.dist',
            'tracker/vendor/phpunit/php-timer/.travis.yml',
            'tracker/vendor/phpunit/php-timer/ChangeLog.md',
            'tracker/vendor/phpunit/php-timer/LICENSE',
            'tracker/vendor/phpunit/php-timer/README.md',
            'tracker/vendor/phpunit/php-timer/build.xml',
            'tracker/vendor/phpunit/php-timer/composer.json',
            'tracker/vendor/phpunit/php-timer/phpunit.xml',
            'tracker/vendor/phpunit/php-timer/src/Exception.php',
            'tracker/vendor/phpunit/php-timer/src/RuntimeException.php',
            'tracker/vendor/phpunit/php-timer/src/Timer.php',
            'tracker/vendor/phpunit/php-timer/tests/TimerTest.php',
            'tracker/vendor/phpunit/php-token-stream/.gitattributes',
            'tracker/vendor/phpunit/php-token-stream/.github/FUNDING.yml',
            'tracker/vendor/phpunit/php-token-stream/.gitignore',
            'tracker/vendor/phpunit/php-token-stream/.travis.yml',
            'tracker/vendor/phpunit/php-token-stream/ChangeLog.md',
            'tracker/vendor/phpunit/php-token-stream/LICENSE',
            'tracker/vendor/phpunit/php-token-stream/README.md',
            'tracker/vendor/phpunit/php-token-stream/build.xml',
            'tracker/vendor/phpunit/php-token-stream/composer.json',
            'tracker/vendor/phpunit/php-token-stream/phpunit.xml',
            'tracker/vendor/phpunit/php-token-stream/src/Token.php',
            'tracker/vendor/phpunit/php-token-stream/src/Token/Stream.php',
            'tracker/vendor/phpunit/php-token-stream/src/Token/Stream/CachingFactory.php',
            'tracker/vendor/phpunit/php-token-stream/src/Token/Util.php',
            'tracker/vendor/phpunit/php-token-stream/tests/Token/ClassTest.php',
            'tracker/vendor/phpunit/php-token-stream/tests/Token/ClosureTest.php',
            'tracker/vendor/phpunit/php-token-stream/tests/Token/FunctionTest.php',
            'tracker/vendor/phpunit/php-token-stream/tests/Token/IncludeTest.php',
            'tracker/vendor/phpunit/php-token-stream/tests/Token/InterfaceTest.php',
            'tracker/vendor/phpunit/php-token-stream/tests/Token/NamespaceTest.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/classExtendsNamespacedClass.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/classInNamespace.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/classInScopedNamespace.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/classUsesNamespacedFunction.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/class_with_method_named_empty.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/class_with_method_that_declares_anonymous_class.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/class_with_method_that_declares_anonymous_class2.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/class_with_multiple_anonymous_classes_and_functions.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/closure.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/issue19.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/issue30.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/multipleNamespacesWithOneClassUsingBraces.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/multipleNamespacesWithOneClassUsingNonBraceSyntax.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/php-code-coverage-issue-424.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/source.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/source2.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/source3.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/source4.php',
            'tracker/vendor/phpunit/php-token-stream/tests/_fixture/source5.php',
            'tracker/vendor/phpunit/php-token-stream/tests/bootstrap.php',
            'tracker/vendor/phpunit/phpunit/.phpstorm.meta.php',
            'tracker/vendor/phpunit/phpunit/ChangeLog-8.5.md',
            'tracker/vendor/phpunit/phpunit/LICENSE',
            'tracker/vendor/phpunit/phpunit/README.md',
            'tracker/vendor/phpunit/phpunit/SECURITY.md',
            'tracker/vendor/phpunit/phpunit/composer.json',
            'tracker/vendor/phpunit/phpunit/phpunit',
            'tracker/vendor/phpunit/phpunit/phpunit.xsd',
            'tracker/vendor/phpunit/phpunit/src/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Assert.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Assert/Functions.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ArrayHasKey.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ArraySubset.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/Attribute.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/Callback.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ClassHasAttribute.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ClassHasStaticAttribute.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/Composite.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/Constraint.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/Count.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/DirectoryExists.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ExceptionCode.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ExceptionMessage.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ExceptionMessageRegularExpression.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/FileExists.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/GreaterThan.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsAnything.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsEmpty.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsEqual.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsFalse.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsFinite.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsIdentical.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsInfinite.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsInstanceOf.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsJson.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsNan.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsNull.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsReadable.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsTrue.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsType.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/IsWritable.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/JsonMatches.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/JsonMatchesErrorMessageProvider.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/LessThan.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/LogicalAnd.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/LogicalNot.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/LogicalOr.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/LogicalXor.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/ObjectHasAttribute.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/RegularExpression.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/SameSize.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/StringContains.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/StringEndsWith.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/StringMatchesFormatDescription.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/StringStartsWith.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/TraversableContains.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/TraversableContainsEqual.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/TraversableContainsIdentical.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Constraint/TraversableContainsOnly.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/DataProviderTestSuite.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Error/Deprecated.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Error/Error.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Error/Notice.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Error/Warning.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/AssertionFailedError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/CodeCoverageException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/CoveredCodeNotExecutedException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/ExpectationFailedException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/IncompleteTestError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/InvalidArgumentException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/InvalidCoversTargetException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/InvalidDataProviderException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/MissingCoversAnnotationException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/NoChildTestSuiteException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/OutputError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/PHPTAssertionFailedError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/RiskyTestError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/SkippedTestError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/SkippedTestSuiteError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/SyntheticError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/SyntheticSkippedError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/UnintentionallyCoveredCodeError.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Exception/Warning.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/ExceptionWrapper.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/IncompleteTest.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/IncompleteTestCase.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/InvalidParameterGroupException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Api/Api.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Api/Method.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/Identity.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/InvocationMocker.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/InvocationStubber.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/Match_.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/MethodNameMatch.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/ParametersMatch.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Builder/Stub.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/ConfigurableMethod.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Exception/BadMethodCallException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Exception/ConfigurableMethodsAlreadyInitializedException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Exception/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Exception/IncompatibleReturnValueException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Exception/RuntimeException.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/deprecation.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/mocked_class.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/mocked_method.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/mocked_method_void.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/mocked_static_method.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/proxied_method.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/proxied_method_void.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/trait_class.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/wsdl_class.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Generator/wsdl_method.tpl',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Invocation.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/InvocationHandler.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Matcher.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MethodNameConstraint.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockBuilder.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockClass.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockMethod.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockMethodSet.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockObject.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockTrait.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/MockType.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/AnyInvokedCount.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/AnyParameters.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/ConsecutiveParameters.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/InvocationOrder.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/InvokedAtIndex.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/InvokedAtLeastCount.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/InvokedAtLeastOnce.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/InvokedAtMostCount.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/InvokedCount.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/MethodName.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/Parameters.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Rule/ParametersRule.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ConsecutiveCalls.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ReturnArgument.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ReturnCallback.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ReturnReference.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ReturnSelf.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ReturnStub.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/ReturnValueMap.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Stub/Stub.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/MockObject/Verifiable.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/SelfDescribing.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/SkippedTest.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/SkippedTestCase.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/Test.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestBuilder.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestCase.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestFailure.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestListener.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestListenerDefaultImplementation.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestResult.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestSuite.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/TestSuiteIterator.php',
            'tracker/vendor/phpunit/phpunit/src/Framework/WarningTestCase.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/BaseTestRunner.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/DefaultTestResultCache.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Filter/ExcludeGroupFilterIterator.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Filter/Factory.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Filter/GroupFilterIterator.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Filter/IncludeGroupFilterIterator.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Filter/NameFilterIterator.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterIncompleteTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterLastTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterRiskyTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterSkippedTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterSuccessfulTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterTestErrorHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterTestFailureHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/AfterTestWarningHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/BeforeFirstTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/BeforeTestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/Hook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/TestHook.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Hook/TestListenerAdapter.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/NullTestResultCache.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/PhptTestCase.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/ResultCacheExtension.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/StandardTestSuiteLoader.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/TestResultCache.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/TestSuiteLoader.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/TestSuiteSorter.php',
            'tracker/vendor/phpunit/phpunit/src/Runner/Version.php',
            'tracker/vendor/phpunit/phpunit/src/TextUI/Command.php',
            'tracker/vendor/phpunit/phpunit/src/TextUI/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/TextUI/Help.php',
            'tracker/vendor/phpunit/phpunit/src/TextUI/ResultPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/TextUI/TestRunner.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Annotation/DocBlock.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Annotation/Registry.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Blacklist.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Color.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Configuration.php',
            'tracker/vendor/phpunit/phpunit/src/Util/ConfigurationGenerator.php',
            'tracker/vendor/phpunit/phpunit/src/Util/ErrorHandler.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Exception.php',
            'tracker/vendor/phpunit/phpunit/src/Util/FileLoader.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Fileloader.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Filesystem.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Filter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Getopt.php',
            'tracker/vendor/phpunit/phpunit/src/Util/GlobalState.php',
            'tracker/vendor/phpunit/phpunit/src/Util/InvalidDataSetException.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Json.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Log/JUnit.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Log/TeamCity.php',
            'tracker/vendor/phpunit/phpunit/src/Util/PHP/AbstractPhpProcess.php',
            'tracker/vendor/phpunit/phpunit/src/Util/PHP/DefaultPhpProcess.php',
            'tracker/vendor/phpunit/phpunit/src/Util/PHP/Template/PhptTestCase.tpl',
            'tracker/vendor/phpunit/phpunit/src/Util/PHP/Template/TestCaseClass.tpl',
            'tracker/vendor/phpunit/phpunit/src/Util/PHP/Template/TestCaseMethod.tpl',
            'tracker/vendor/phpunit/phpunit/src/Util/PHP/WindowsPhpProcess.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Printer.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Reflection.php',
            'tracker/vendor/phpunit/phpunit/src/Util/RegularExpression.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Test.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/CliTestDoxPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/HtmlResultPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/NamePrettifier.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/ResultPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/TestDoxPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/TextResultPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TestDox/XmlResultPrinter.php',
            'tracker/vendor/phpunit/phpunit/src/Util/TextTestListRenderer.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Type.php',
            'tracker/vendor/phpunit/phpunit/src/Util/VersionComparisonOperator.php',
            'tracker/vendor/phpunit/phpunit/src/Util/XML.php',
            'tracker/vendor/phpunit/phpunit/src/Util/XdebugFilterScriptGenerator.php',
            'tracker/vendor/phpunit/phpunit/src/Util/Xml.php',
            'tracker/vendor/phpunit/phpunit/src/Util/XmlTestListRenderer.php',
            'tracker/vendor/pimple/pimple/.github/workflows/tests.yml',
            'tracker/vendor/pimple/pimple/.gitignore',
            'tracker/vendor/pimple/pimple/.php_cs.dist',
            'tracker/vendor/pimple/pimple/CHANGELOG',
            'tracker/vendor/pimple/pimple/LICENSE',
            'tracker/vendor/pimple/pimple/README.rst',
            'tracker/vendor/pimple/pimple/composer.json',
            'tracker/vendor/pimple/pimple/phpunit.xml.dist',
            'tracker/vendor/pimple/pimple/src/Pimple/Container.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Exception/ExpectedInvokableException.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Exception/FrozenServiceException.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Exception/InvalidServiceIdentifierException.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Exception/UnknownIdentifierException.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Psr11/Container.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Psr11/ServiceLocator.php',
            'tracker/vendor/pimple/pimple/src/Pimple/ServiceIterator.php',
            'tracker/vendor/pimple/pimple/src/Pimple/ServiceProviderInterface.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/Fixtures/Invokable.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/Fixtures/NonInvokable.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/Fixtures/PimpleServiceProvider.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/Fixtures/Service.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/PimpleServiceProviderInterfaceTest.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/PimpleTest.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/Psr11/ContainerTest.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/Psr11/ServiceLocatorTest.php',
            'tracker/vendor/pimple/pimple/src/Pimple/Tests/ServiceIteratorTest.php',
            'tracker/vendor/psr/container/.gitignore',
            'tracker/vendor/psr/container/LICENSE',
            'tracker/vendor/psr/container/README.md',
            'tracker/vendor/psr/container/composer.json',
            'tracker/vendor/psr/container/src/ContainerExceptionInterface.php',
            'tracker/vendor/psr/container/src/ContainerInterface.php',
            'tracker/vendor/psr/container/src/NotFoundExceptionInterface.php',
            'tracker/vendor/psr/http-client/CHANGELOG.md',
            'tracker/vendor/psr/http-client/LICENSE',
            'tracker/vendor/psr/http-client/README.md',
            'tracker/vendor/psr/http-client/composer.json',
            'tracker/vendor/psr/http-client/src/ClientExceptionInterface.php',
            'tracker/vendor/psr/http-client/src/ClientInterface.php',
            'tracker/vendor/psr/http-client/src/NetworkExceptionInterface.php',
            'tracker/vendor/psr/http-client/src/RequestExceptionInterface.php',
            'tracker/vendor/psr/http-factory/LICENSE',
            'tracker/vendor/psr/http-factory/README.md',
            'tracker/vendor/psr/http-factory/composer.json',
            'tracker/vendor/psr/http-factory/src/RequestFactoryInterface.php',
            'tracker/vendor/psr/http-factory/src/ResponseFactoryInterface.php',
            'tracker/vendor/psr/http-factory/src/ServerRequestFactoryInterface.php',
            'tracker/vendor/psr/http-factory/src/StreamFactoryInterface.php',
            'tracker/vendor/psr/http-factory/src/UploadedFileFactoryInterface.php',
            'tracker/vendor/psr/http-factory/src/UriFactoryInterface.php',
            'tracker/vendor/psr/http-message/CHANGELOG.md',
            'tracker/vendor/psr/http-message/LICENSE',
            'tracker/vendor/psr/http-message/README.md',
            'tracker/vendor/psr/http-message/composer.json',
            'tracker/vendor/psr/http-message/docs/PSR7-Interfaces.md',
            'tracker/vendor/psr/http-message/docs/PSR7-Usage.md',
            'tracker/vendor/psr/http-message/src/MessageInterface.php',
            'tracker/vendor/psr/http-message/src/RequestInterface.php',
            'tracker/vendor/psr/http-message/src/ResponseInterface.php',
            'tracker/vendor/psr/http-message/src/ServerRequestInterface.php',
            'tracker/vendor/psr/http-message/src/StreamInterface.php',
            'tracker/vendor/psr/http-message/src/UploadedFileInterface.php',
            'tracker/vendor/psr/http-message/src/UriInterface.php',
            'tracker/vendor/ralouphie/getallheaders/LICENSE',
            'tracker/vendor/ralouphie/getallheaders/README.md',
            'tracker/vendor/ralouphie/getallheaders/composer.json',
            'tracker/vendor/ralouphie/getallheaders/src/getallheaders.php',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/.gitignore',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/.php_cs',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/.travis.yml',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/ChangeLog.md',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/LICENSE',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/README.md',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/build.xml',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/composer.json',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/phpunit.xml',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/src/Wizard.php',
            'tracker/vendor/sebastian/code-unit-reverse-lookup/tests/WizardTest.php',
            'tracker/vendor/sebastian/comparator/ChangeLog.md',
            'tracker/vendor/sebastian/comparator/LICENSE',
            'tracker/vendor/sebastian/comparator/README.md',
            'tracker/vendor/sebastian/comparator/composer.json',
            'tracker/vendor/sebastian/comparator/src/ArrayComparator.php',
            'tracker/vendor/sebastian/comparator/src/Comparator.php',
            'tracker/vendor/sebastian/comparator/src/ComparisonFailure.php',
            'tracker/vendor/sebastian/comparator/src/DOMNodeComparator.php',
            'tracker/vendor/sebastian/comparator/src/DateTimeComparator.php',
            'tracker/vendor/sebastian/comparator/src/DoubleComparator.php',
            'tracker/vendor/sebastian/comparator/src/ExceptionComparator.php',
            'tracker/vendor/sebastian/comparator/src/Factory.php',
            'tracker/vendor/sebastian/comparator/src/MockObjectComparator.php',
            'tracker/vendor/sebastian/comparator/src/NumericComparator.php',
            'tracker/vendor/sebastian/comparator/src/ObjectComparator.php',
            'tracker/vendor/sebastian/comparator/src/ResourceComparator.php',
            'tracker/vendor/sebastian/comparator/src/ScalarComparator.php',
            'tracker/vendor/sebastian/comparator/src/SplObjectStorageComparator.php',
            'tracker/vendor/sebastian/comparator/src/TypeComparator.php',
            'tracker/vendor/sebastian/diff/.github/stale.yml',
            'tracker/vendor/sebastian/diff/.gitignore',
            'tracker/vendor/sebastian/diff/.php_cs.dist',
            'tracker/vendor/sebastian/diff/.travis.yml',
            'tracker/vendor/sebastian/diff/ChangeLog.md',
            'tracker/vendor/sebastian/diff/LICENSE',
            'tracker/vendor/sebastian/diff/README.md',
            'tracker/vendor/sebastian/diff/build.xml',
            'tracker/vendor/sebastian/diff/composer.json',
            'tracker/vendor/sebastian/diff/phpunit.xml',
            'tracker/vendor/sebastian/diff/src/Chunk.php',
            'tracker/vendor/sebastian/diff/src/Diff.php',
            'tracker/vendor/sebastian/diff/src/Differ.php',
            'tracker/vendor/sebastian/diff/src/Exception/ConfigurationException.php',
            'tracker/vendor/sebastian/diff/src/Exception/Exception.php',
            'tracker/vendor/sebastian/diff/src/Exception/InvalidArgumentException.php',
            'tracker/vendor/sebastian/diff/src/Line.php',
            'tracker/vendor/sebastian/diff/src/LongestCommonSubsequenceCalculator.php',
            'tracker/vendor/sebastian/diff/src/MemoryEfficientLongestCommonSubsequenceCalculator.php',
            'tracker/vendor/sebastian/diff/src/Output/AbstractChunkOutputBuilder.php',
            'tracker/vendor/sebastian/diff/src/Output/DiffOnlyOutputBuilder.php',
            'tracker/vendor/sebastian/diff/src/Output/DiffOutputBuilderInterface.php',
            'tracker/vendor/sebastian/diff/src/Output/StrictUnifiedDiffOutputBuilder.php',
            'tracker/vendor/sebastian/diff/src/Output/UnifiedDiffOutputBuilder.php',
            'tracker/vendor/sebastian/diff/src/Parser.php',
            'tracker/vendor/sebastian/diff/src/TimeEfficientLongestCommonSubsequenceCalculator.php',
            'tracker/vendor/sebastian/diff/tests/ChunkTest.php',
            'tracker/vendor/sebastian/diff/tests/DiffTest.php',
            'tracker/vendor/sebastian/diff/tests/DifferTest.php',
            'tracker/vendor/sebastian/diff/tests/Exception/ConfigurationExceptionTest.php',
            'tracker/vendor/sebastian/diff/tests/Exception/InvalidArgumentExceptionTest.php',
            'tracker/vendor/sebastian/diff/tests/LineTest.php',
            'tracker/vendor/sebastian/diff/tests/LongestCommonSubsequenceTest.php',
            'tracker/vendor/sebastian/diff/tests/MemoryEfficientImplementationTest.php',
            'tracker/vendor/sebastian/diff/tests/Output/AbstractChunkOutputBuilderTest.php',
            'tracker/vendor/sebastian/diff/tests/Output/DiffOnlyOutputBuilderTest.php',
            'tracker/vendor/sebastian/diff/tests/Output/Integration/StrictUnifiedDiffOutputBuilderIntegrationTest.php',
            'tracker/vendor/sebastian/diff/tests/Output/Integration/UnifiedDiffOutputBuilderIntegrationTest.php',
            'tracker/vendor/sebastian/diff/tests/Output/StrictUnifiedDiffOutputBuilderDataProvider.php',
            'tracker/vendor/sebastian/diff/tests/Output/StrictUnifiedDiffOutputBuilderTest.php',
            'tracker/vendor/sebastian/diff/tests/Output/UnifiedDiffOutputBuilderDataProvider.php',
            'tracker/vendor/sebastian/diff/tests/Output/UnifiedDiffOutputBuilderTest.php',
            'tracker/vendor/sebastian/diff/tests/ParserTest.php',
            'tracker/vendor/sebastian/diff/tests/TimeEfficientImplementationTest.php',
            'tracker/vendor/sebastian/diff/tests/Utils/FileUtils.php',
            'tracker/vendor/sebastian/diff/tests/Utils/UnifiedDiffAssertTrait.php',
            'tracker/vendor/sebastian/diff/tests/Utils/UnifiedDiffAssertTraitIntegrationTest.php',
            'tracker/vendor/sebastian/diff/tests/Utils/UnifiedDiffAssertTraitTest.php',
            'tracker/vendor/sebastian/diff/tests/fixtures/.editorconfig',
            'tracker/vendor/sebastian/diff/tests/fixtures/UnifiedDiffAssertTraitIntegrationTest/1_a.txt',
            'tracker/vendor/sebastian/diff/tests/fixtures/UnifiedDiffAssertTraitIntegrationTest/1_b.txt',
            'tracker/vendor/sebastian/diff/tests/fixtures/UnifiedDiffAssertTraitIntegrationTest/2_a.txt',
            'tracker/vendor/sebastian/diff/tests/fixtures/UnifiedDiffAssertTraitIntegrationTest/2_b.txt',
            'tracker/vendor/sebastian/diff/tests/fixtures/out/.editorconfig',
            'tracker/vendor/sebastian/diff/tests/fixtures/out/.gitignore',
            'tracker/vendor/sebastian/diff/tests/fixtures/patch.txt',
            'tracker/vendor/sebastian/diff/tests/fixtures/patch2.txt',
            'tracker/vendor/sebastian/diff/tests/fixtures/serialized_diff.bin',
            'tracker/vendor/sebastian/environment/.github/FUNDING.yml',
            'tracker/vendor/sebastian/environment/.gitignore',
            'tracker/vendor/sebastian/environment/.php_cs.dist',
            'tracker/vendor/sebastian/environment/.travis.yml',
            'tracker/vendor/sebastian/environment/ChangeLog.md',
            'tracker/vendor/sebastian/environment/LICENSE',
            'tracker/vendor/sebastian/environment/README.md',
            'tracker/vendor/sebastian/environment/build.xml',
            'tracker/vendor/sebastian/environment/composer.json',
            'tracker/vendor/sebastian/environment/phpunit.xml',
            'tracker/vendor/sebastian/environment/src/Console.php',
            'tracker/vendor/sebastian/environment/src/OperatingSystem.php',
            'tracker/vendor/sebastian/environment/src/Runtime.php',
            'tracker/vendor/sebastian/environment/tests/ConsoleTest.php',
            'tracker/vendor/sebastian/environment/tests/OperatingSystemTest.php',
            'tracker/vendor/sebastian/environment/tests/RuntimeTest.php',
            'tracker/vendor/sebastian/exporter/ChangeLog.md',
            'tracker/vendor/sebastian/exporter/LICENSE',
            'tracker/vendor/sebastian/exporter/README.md',
            'tracker/vendor/sebastian/exporter/composer.json',
            'tracker/vendor/sebastian/exporter/src/Exporter.php',
            'tracker/vendor/sebastian/global-state/.github/stale.yml',
            'tracker/vendor/sebastian/global-state/.gitignore',
            'tracker/vendor/sebastian/global-state/.php_cs.dist',
            'tracker/vendor/sebastian/global-state/.travis.yml',
            'tracker/vendor/sebastian/global-state/ChangeLog.md',
            'tracker/vendor/sebastian/global-state/LICENSE',
            'tracker/vendor/sebastian/global-state/README.md',
            'tracker/vendor/sebastian/global-state/build.xml',
            'tracker/vendor/sebastian/global-state/composer.json',
            'tracker/vendor/sebastian/global-state/phpunit.xml',
            'tracker/vendor/sebastian/global-state/src/Blacklist.php',
            'tracker/vendor/sebastian/global-state/src/CodeExporter.php',
            'tracker/vendor/sebastian/global-state/src/Restorer.php',
            'tracker/vendor/sebastian/global-state/src/Snapshot.php',
            'tracker/vendor/sebastian/global-state/src/exceptions/Exception.php',
            'tracker/vendor/sebastian/global-state/src/exceptions/RuntimeException.php',
            'tracker/vendor/sebastian/global-state/tests/BlacklistTest.php',
            'tracker/vendor/sebastian/global-state/tests/CodeExporterTest.php',
            'tracker/vendor/sebastian/global-state/tests/RestorerTest.php',
            'tracker/vendor/sebastian/global-state/tests/SnapshotTest.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/BlacklistedChildClass.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/BlacklistedClass.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/BlacklistedImplementor.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/BlacklistedInterface.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/SnapshotClass.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/SnapshotDomDocument.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/SnapshotFunctions.php',
            'tracker/vendor/sebastian/global-state/tests/_fixture/SnapshotTrait.php',
            'tracker/vendor/sebastian/object-enumerator/.gitignore',
            'tracker/vendor/sebastian/object-enumerator/.php_cs',
            'tracker/vendor/sebastian/object-enumerator/.travis.yml',
            'tracker/vendor/sebastian/object-enumerator/ChangeLog.md',
            'tracker/vendor/sebastian/object-enumerator/LICENSE',
            'tracker/vendor/sebastian/object-enumerator/README.md',
            'tracker/vendor/sebastian/object-enumerator/build.xml',
            'tracker/vendor/sebastian/object-enumerator/composer.json',
            'tracker/vendor/sebastian/object-enumerator/phpunit.xml',
            'tracker/vendor/sebastian/object-enumerator/src/Enumerator.php',
            'tracker/vendor/sebastian/object-enumerator/src/Exception.php',
            'tracker/vendor/sebastian/object-enumerator/src/InvalidArgumentException.php',
            'tracker/vendor/sebastian/object-enumerator/tests/EnumeratorTest.php',
            'tracker/vendor/sebastian/object-enumerator/tests/_fixture/ExceptionThrower.php',
            'tracker/vendor/sebastian/object-reflector/.gitignore',
            'tracker/vendor/sebastian/object-reflector/.php_cs',
            'tracker/vendor/sebastian/object-reflector/.travis.yml',
            'tracker/vendor/sebastian/object-reflector/ChangeLog.md',
            'tracker/vendor/sebastian/object-reflector/LICENSE',
            'tracker/vendor/sebastian/object-reflector/README.md',
            'tracker/vendor/sebastian/object-reflector/build.xml',
            'tracker/vendor/sebastian/object-reflector/composer.json',
            'tracker/vendor/sebastian/object-reflector/phpunit.xml',
            'tracker/vendor/sebastian/object-reflector/src/Exception.php',
            'tracker/vendor/sebastian/object-reflector/src/InvalidArgumentException.php',
            'tracker/vendor/sebastian/object-reflector/src/ObjectReflector.php',
            'tracker/vendor/sebastian/object-reflector/tests/ObjectReflectorTest.php',
            'tracker/vendor/sebastian/object-reflector/tests/_fixture/ChildClass.php',
            'tracker/vendor/sebastian/object-reflector/tests/_fixture/ClassWithIntegerAttributeName.php',
            'tracker/vendor/sebastian/object-reflector/tests/_fixture/ParentClass.php',
            'tracker/vendor/sebastian/recursion-context/.gitignore',
            'tracker/vendor/sebastian/recursion-context/.travis.yml',
            'tracker/vendor/sebastian/recursion-context/LICENSE',
            'tracker/vendor/sebastian/recursion-context/README.md',
            'tracker/vendor/sebastian/recursion-context/build.xml',
            'tracker/vendor/sebastian/recursion-context/composer.json',
            'tracker/vendor/sebastian/recursion-context/phpunit.xml',
            'tracker/vendor/sebastian/recursion-context/src/Context.php',
            'tracker/vendor/sebastian/recursion-context/src/Exception.php',
            'tracker/vendor/sebastian/recursion-context/src/InvalidArgumentException.php',
            'tracker/vendor/sebastian/recursion-context/tests/ContextTest.php',
            'tracker/vendor/sebastian/resource-operations/.github/stale.yml',
            'tracker/vendor/sebastian/resource-operations/.gitignore',
            'tracker/vendor/sebastian/resource-operations/.php_cs.dist',
            'tracker/vendor/sebastian/resource-operations/ChangeLog.md',
            'tracker/vendor/sebastian/resource-operations/LICENSE',
            'tracker/vendor/sebastian/resource-operations/README.md',
            'tracker/vendor/sebastian/resource-operations/build.xml',
            'tracker/vendor/sebastian/resource-operations/build/generate.php',
            'tracker/vendor/sebastian/resource-operations/composer.json',
            'tracker/vendor/sebastian/resource-operations/src/ResourceOperations.php',
            'tracker/vendor/sebastian/resource-operations/tests/ResourceOperationsTest.php',
            'tracker/vendor/sebastian/type/.gitattributes',
            'tracker/vendor/sebastian/type/.github/FUNDING.yml',
            'tracker/vendor/sebastian/type/.gitignore',
            'tracker/vendor/sebastian/type/.php_cs.dist',
            'tracker/vendor/sebastian/type/.travis.yml',
            'tracker/vendor/sebastian/type/ChangeLog.md',
            'tracker/vendor/sebastian/type/LICENSE',
            'tracker/vendor/sebastian/type/README.md',
            'tracker/vendor/sebastian/type/build.xml',
            'tracker/vendor/sebastian/type/composer.json',
            'tracker/vendor/sebastian/type/phive.xml',
            'tracker/vendor/sebastian/type/phpunit.xml',
            'tracker/vendor/sebastian/type/psalm.xml',
            'tracker/vendor/sebastian/type/src/CallableType.php',
            'tracker/vendor/sebastian/type/src/GenericObjectType.php',
            'tracker/vendor/sebastian/type/src/IterableType.php',
            'tracker/vendor/sebastian/type/src/NullType.php',
            'tracker/vendor/sebastian/type/src/ObjectType.php',
            'tracker/vendor/sebastian/type/src/SimpleType.php',
            'tracker/vendor/sebastian/type/src/Type.php',
            'tracker/vendor/sebastian/type/src/TypeName.php',
            'tracker/vendor/sebastian/type/src/UnknownType.php',
            'tracker/vendor/sebastian/type/src/VoidType.php',
            'tracker/vendor/sebastian/type/src/exception/Exception.php',
            'tracker/vendor/sebastian/type/src/exception/RuntimeException.php',
            'tracker/vendor/sebastian/type/tests/_fixture/ChildClass.php',
            'tracker/vendor/sebastian/type/tests/_fixture/ClassWithCallbackMethods.php',
            'tracker/vendor/sebastian/type/tests/_fixture/ClassWithInvokeMethod.php',
            'tracker/vendor/sebastian/type/tests/_fixture/Iterator.php',
            'tracker/vendor/sebastian/type/tests/_fixture/ParentClass.php',
            'tracker/vendor/sebastian/type/tests/_fixture/callback_function.php',
            'tracker/vendor/sebastian/type/tests/unit/CallableTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/GenericObjectTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/IterableTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/NullTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/ObjectTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/SimpleTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/TypeNameTest.php',
            'tracker/vendor/sebastian/type/tests/unit/TypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/UnknownTypeTest.php',
            'tracker/vendor/sebastian/type/tests/unit/VoidTypeTest.php',
            'tracker/vendor/sebastian/version/.gitattributes',
            'tracker/vendor/sebastian/version/.gitignore',
            'tracker/vendor/sebastian/version/.php_cs',
            'tracker/vendor/sebastian/version/LICENSE',
            'tracker/vendor/sebastian/version/README.md',
            'tracker/vendor/sebastian/version/composer.json',
            'tracker/vendor/sebastian/version/src/Version.php',
            'tracker/vendor/slim/slim/LICENSE.md',
            'tracker/vendor/slim/slim/MAINTAINERS.md',
            'tracker/vendor/slim/slim/Slim/App.php',
            'tracker/vendor/slim/slim/Slim/CallableResolver.php',
            'tracker/vendor/slim/slim/Slim/CallableResolverAwareTrait.php',
            'tracker/vendor/slim/slim/Slim/Collection.php',
            'tracker/vendor/slim/slim/Slim/Container.php',
            'tracker/vendor/slim/slim/Slim/DefaultServicesProvider.php',
            'tracker/vendor/slim/slim/Slim/DeferredCallable.php',
            'tracker/vendor/slim/slim/Slim/Exception/ContainerException.php',
            'tracker/vendor/slim/slim/Slim/Exception/ContainerValueNotFoundException.php',
            'tracker/vendor/slim/slim/Slim/Exception/InvalidMethodException.php',
            'tracker/vendor/slim/slim/Slim/Exception/MethodNotAllowedException.php',
            'tracker/vendor/slim/slim/Slim/Exception/NotFoundException.php',
            'tracker/vendor/slim/slim/Slim/Exception/SlimException.php',
            'tracker/vendor/slim/slim/Slim/Handlers/AbstractError.php',
            'tracker/vendor/slim/slim/Slim/Handlers/AbstractHandler.php',
            'tracker/vendor/slim/slim/Slim/Handlers/Error.php',
            'tracker/vendor/slim/slim/Slim/Handlers/NotAllowed.php',
            'tracker/vendor/slim/slim/Slim/Handlers/NotFound.php',
            'tracker/vendor/slim/slim/Slim/Handlers/PhpError.php',
            'tracker/vendor/slim/slim/Slim/Handlers/Strategies/RequestResponse.php',
            'tracker/vendor/slim/slim/Slim/Handlers/Strategies/RequestResponseArgs.php',
            'tracker/vendor/slim/slim/Slim/Http/Body.php',
            'tracker/vendor/slim/slim/Slim/Http/Cookies.php',
            'tracker/vendor/slim/slim/Slim/Http/Environment.php',
            'tracker/vendor/slim/slim/Slim/Http/Headers.php',
            'tracker/vendor/slim/slim/Slim/Http/Message.php',
            'tracker/vendor/slim/slim/Slim/Http/NonBufferedBody.php',
            'tracker/vendor/slim/slim/Slim/Http/Request.php',
            'tracker/vendor/slim/slim/Slim/Http/RequestBody.php',
            'tracker/vendor/slim/slim/Slim/Http/Response.php',
            'tracker/vendor/slim/slim/Slim/Http/StatusCode.php',
            'tracker/vendor/slim/slim/Slim/Http/Stream.php',
            'tracker/vendor/slim/slim/Slim/Http/UploadedFile.php',
            'tracker/vendor/slim/slim/Slim/Http/Uri.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/CallableResolverInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/CollectionInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/Http/CookiesInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/Http/EnvironmentInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/Http/HeadersInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/InvocationStrategyInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/RouteGroupInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/RouteInterface.php',
            'tracker/vendor/slim/slim/Slim/Interfaces/RouterInterface.php',
            'tracker/vendor/slim/slim/Slim/MiddlewareAwareTrait.php',
            'tracker/vendor/slim/slim/Slim/Routable.php',
            'tracker/vendor/slim/slim/Slim/Route.php',
            'tracker/vendor/slim/slim/Slim/RouteGroup.php',
            'tracker/vendor/slim/slim/Slim/Router.php',
            'tracker/vendor/slim/slim/composer.json',
            'tracker/vendor/slim/slim/phpstan.neon.dist',
            'tracker/vendor/symfony/deprecation-contracts/.gitignore',
            'tracker/vendor/symfony/deprecation-contracts/CHANGELOG.md',
            'tracker/vendor/symfony/deprecation-contracts/LICENSE',
            'tracker/vendor/symfony/deprecation-contracts/README.md',
            'tracker/vendor/symfony/deprecation-contracts/composer.json',
            'tracker/vendor/symfony/deprecation-contracts/function.php',
            'tracker/vendor/theseer/tokenizer/.php_cs.dist',
            'tracker/vendor/theseer/tokenizer/CHANGELOG.md',
            'tracker/vendor/theseer/tokenizer/LICENSE',
            'tracker/vendor/theseer/tokenizer/README.md',
            'tracker/vendor/theseer/tokenizer/composer.json',
            'tracker/vendor/theseer/tokenizer/composer.lock',
            'tracker/vendor/theseer/tokenizer/src/Exception.php',
            'tracker/vendor/theseer/tokenizer/src/NamespaceUri.php',
            'tracker/vendor/theseer/tokenizer/src/NamespaceUriException.php',
            'tracker/vendor/theseer/tokenizer/src/Token.php',
            'tracker/vendor/theseer/tokenizer/src/TokenCollection.php',
            'tracker/vendor/theseer/tokenizer/src/TokenCollectionException.php',
            'tracker/vendor/theseer/tokenizer/src/Tokenizer.php',
            'tracker/vendor/theseer/tokenizer/src/XMLSerializer.php',
            'tracker/vendor/vboctor/disposable_email_checker/.gitignore',
            'tracker/vendor/vboctor/disposable_email_checker/.travis.yml',
            'tracker/vendor/vboctor/disposable_email_checker/changelog.txt',
            'tracker/vendor/vboctor/disposable_email_checker/composer.json',
            'tracker/vendor/vboctor/disposable_email_checker/composer.lock',
            'tracker/vendor/vboctor/disposable_email_checker/data/.htaccess',
            'tracker/vendor/vboctor/disposable_email_checker/data/domains.txt',
            'tracker/vendor/vboctor/disposable_email_checker/license.txt',
            'tracker/vendor/vboctor/disposable_email_checker/php/src/DisposableEmailChecker.php',
            'tracker/vendor/vboctor/disposable_email_checker/php/tests/disposable_email_checker_tests.php',
            'tracker/vendor/vboctor/disposable_email_checker/readme.md',
            'tracker/verify.php',
            'tracker/view.php',
            'tracker/view_all_bug_page.php',
            'tracker/view_all_inc.php',
            'tracker/view_all_set.php',
            'tracker/view_filters_page.php',
            'tracker/view_user_page.php',
            'tracker/wiki.php',
            'tracker/workflow_graph_img.php',
            'tracker/xmlhttprequest.php',
        ];
    }
}
